function CameraUFXCTFigure(outputFolder,figureParameters)

% Creates a graph using the data from the file data.mat in the output
% folder.
% All necessary parameters are read from the file figureParameters, found
% in the output folder.

% Theodoros-Nestor Papapetrou
% Last edited: 15 Mar 2023

%% Input

% outputFolder: the path to the output folder, where the generated figure
    % is saved. The output folder needs to exist and contain the 
    % figureParameters file. The sourceFolder, given in figureParameters,
    % must contain a 'data.mat' file created by the function
    % CameraUFXCTProcessingDyn3 or by the function UFXCTProcessingFul4.
    % e.g. outputFolder = 'P:\Camera_ROFEX_data\f0p50';
    
% figureParameters: the name of a *.m file in the outputFolder, containing
    % all parameters needed to make the figure.
    % e.g. figureParameters = 'figureParameters1.m';

%% Read parameters

run(fullfile(outputFolder,figureParameters));

%% Load data

data = fullfile(sourceFolder,'data.mat');
load(data,'runData');

%% Extract key variable values from runData

runDataKeyValues = extractKeyValues(runData,key);

%% Convert data info to data series

% Scatter series
[xs,ys] = findDataSeries(runData,scatterDataInfo,runDataKeyValues);

% Plot series
[xp,yp] = findDataSeries(runData,plotDataInfo,runDataKeyValues);

% Errorbar series
[xe,ye,yerre] = findDataSeries(runData,errorbarDataInfo,runDataKeyValues);

%% Create axes

figure
hold on
ax = gca;
ax.TickLabelInterpreter = 'latex';
if exist('fontsize','var')
    ax.FontSize = fontsize;
end
if exist('scalex','var')
    ax.XScale = scalex;
end
if exist('scaley','var')
    ax.YScale = scaley;
end
if exist('limx','var')
    ax.XLim = limx;
end
if exist('limy','var')
    ax.YLim = limy;
end
if exist('ticksx','var')
    ax.XTick = ticksx;
end
if exist('ticksy','var')
    ax.YTick = ticksy;
end
if exist('ticklabelsx','var')
    ax.XTickLabel = ticklabelsx;
end
if exist('ticklabelsy','var')
    ax.YTickLabel = ticklabelsy;
end
xlabel(labelx,'Interpreter','Latex');
ylabel(labely,'Interpreter','Latex');

%% Add data series

noOfScatterSeries = numel(xs);
noOfPlotSeries = numel(xp);
noOfErrorbarSeries = numel(xe);
noOfSeries = noOfScatterSeries + noOfPlotSeries + noOfErrorbarSeries;
scatterCurrent = 0;
plotCurrent = 0;
errorbarCurrent = 0;

for ii = 1:noOfSeries
    
    if isequal(sequence{ii},'s') % scatter
        
        scatterCurrent = scatterCurrent + 1;
        jj = scatterCurrent;
        s{jj} = scatter(xs{jj},ys{jj},'HandleVisibility',hv{ii});
        
    elseif isequal(sequence{ii},'p') % plot
        
        plotCurrent = plotCurrent + 1;
        jj = plotCurrent;
        p{jj} = plot(xp{jj},yp{jj},'HandleVisibility',hv{ii});
        
    else % errorbar
        
        errorbarCurrent = errorbarCurrent + 1;
        jj = errorbarCurrent;
        e{jj} = errorbar(xe{jj},ye{jj},yerre{jj},'HandleVisibility',hv{ii});
        
    end
    
end

%% Format scatter data series

% marker size
if exist('sMsz','var')
    for ii = 1:noOfScatterSeries
        s{ii}.SizeData = sMsz(ii);
    end
end

% marker line width
if exist('sMlw','var')
    for ii = 1:noOfScatterSeries
        s{ii}.LineWidth = sMlw(ii);
    end
end

% marker shape
if exist('sMsh','var')
    for ii = 1:noOfScatterSeries
        s{ii}.Marker = sMsh{ii};
    end
end

% marker line color
if exist('sMlc','var')
    for ii = 1:noOfScatterSeries
        s{ii}.MarkerEdgeColor = sMlc{ii};
    end
end

% marker fill color
if exist('sMfc','var')
    for ii = 1:noOfScatterSeries
        s{ii}.MarkerFaceColor = sMfc{ii};
    end
end

% marker line transparency
if exist('sMlt','var')
    for ii = 1:noOfScatterSeries
        s{ii}.MarkerEdgeAlpha = sMlt(ii);
    end
end

% marker fill transparency
if exist('sMft','var')
    for ii = 1:noOfScatterSeries
        s{ii}.MarkerFaceAlpha = sMft(ii);
    end
end

%% Format plot data series

% line width
if exist('pLwd','var')
    for ii = 1:noOfPlotSeries
        p{ii}.LineWidth = pLwd(ii);
    end
end

% line style
if exist('pLst','var')
    for ii = 1:noOfPlotSeries
        p{ii}.LineStyle = pLst{ii};
    end
end

% line color
if exist('pLcl','var')
    for ii = 1:noOfPlotSeries
        p{ii}.Color = pLcl{ii};
    end
end

%% Format errorbar data series

% line style (line connecting the markers)
if exist('eLst','var')
    for ii = 1:noOfErrorbarSeries
        e{ii}.LineStyle = eLst{ii};
    end
end

% line color
if exist('eLcl','var')
    for ii = 1:noOfErrorbarSeries
        e{ii}.Color = eLcl{ii};
    end
end

% marker size
if exist('eMsz','var')
    for ii = 1:noOfErrorbarSeries
        e{ii}.MarkerSize = eMsz(ii);
    end
end

% marker line width
if exist('eMlw','var')
    for ii = 1:noOfErrorbarSeries
        e{ii}.LineWidth = eMlw(ii);
    end
end

% marker shape
if exist('eMsh','var')
    for ii = 1:noOfErrorbarSeries
        e{ii}.Marker = eMsh{ii};
    end
end

% marker line color
if exist('eMlc','var')
    for ii = 1:noOfErrorbarSeries
        e{ii}.MarkerEdgeColor = eMlc{ii};
    end
end

% marker fill color
if exist('eMfc','var')
    for ii = 1:noOfErrorbarSeries
        e{ii}.MarkerFaceColor = eMfc{ii};
    end
end

% cap size (at the end of error bars)
if exist('eCsz','var')
    for ii = 1:noOfErrorbarSeries
        e{ii}.CapSize = eCsz(ii);
    end
end
% eCsz = [15 15 15];

%% Add legend

if exist('label','var')
    lgd = legend(label,'Interpreter','latex','Location','best');
    if exist('fontsize','var')
        lgd.FontSize = fontsize;
    end
end

%% Save figure

saveas(ax,fullfile(outputFolder,figureName));
close

end

function runDataKeyValues = extractKeyValues(runData,key)

noOfKeys = numel(key);
noOfRuns = numel(runData);
runDataKeyValues = zeros(noOfRuns,noOfKeys);

for kk = 1:noOfKeys
    if strcmpi(key{kk},'f')
        runDataKeyValues(:,kk) = [runData.fillingDegree];
    elseif strcmpi(key{kk},'w')
        runDataKeyValues(:,kk) = [runData.rotationalSpeed_rpm];
    elseif strcmpi(key{kk},'z')
        runDataKeyValues(:,kk) = [runData.depth_cm];
    elseif strcmpi(key{kk},'r')
        runDataKeyValues(:,kk) = [runData.rotations];
    else
        error(['Unknown key variable: ' key{kk} ' .'])
    end
end

end

function [x,y,yerr] = findDataSeries(runData,dataInfo,runDataKeyValues)

noOfDataSeries = size(dataInfo,1);
x = cell(1,noOfDataSeries);
y = cell(1,noOfDataSeries);
yerr = cell(1,noOfDataSeries);
DataPerSeries = size(dataInfo,2) - 1;
yerrFind = false;
if DataPerSeries > 2
    yerrFind = true;
end

for ds = 1:noOfDataSeries
    
    seriesInfo = dataInfo(ds,:);
    
    % Check if any of the xInfo, yInfo or yerrInfo are strings, and limit
    % runData if so
    runDataLim = [];
    for ii = 1:DataPerSeries
        if ischar(seriesInfo{ii})
            keyInfo = seriesInfo{end};
            runDataLim = limitRunData(runData,runDataKeyValues,keyInfo);
            break
        end
    end   
    
    % assign x-data values
    x{ds} = assignValues(seriesInfo{1},ds,runDataLim);
    
    % assign y-data values
    y{ds} = assignValues(seriesInfo{2},ds,runDataLim);
    
    % check if x- and y-data are compatible
    [x{ds},y{ds}] = checkSizeCompatibility(x{ds},y{ds},ds);
    
    if yerrFind
        
        % assign yerr-data values
        yerr{ds} = assignValues(seriesInfo{3},ds,runDataLim);
        
        % check if y- and yerr-data are compatible
        [yerr{ds},y{ds}] = checkSizeCompatibility(yerr{ds},y{ds},ds);
        
    end
    
end

end

function val = assignValues(valInfo,ds,runDataLim)

if ischar(valInfo)
    val = extractDataValues(runDataLim,valInfo);
elseif isnumeric(valInfo)
    val = valInfo;
else
    error(['Unknown information: data series no. ' num2str(ds) ' .'])
end

end

function [x,y] = checkSizeCompatibility(x,y,ds)

if numel(x) ~= numel(y)
    if min(numel(x),numel(y)) == 1
        if numel(x) == 1
            x = x*ones(size(y));
        else
            y = y*ones(size(x));
        end
    else
        error(['Data series no. ' num2str(ds) ': size incompatibility.'])
    end
end

end

function runDataLim = limitRunData(runData,runDataKeyValues,keyInfo)

[noOfRuns,noOfKeys] = size(runDataKeyValues);
idxRun = false(1,noOfKeys);
idx = false(1,noOfRuns);

for rr = 1:noOfRuns
    for kk = 1:noOfKeys
        idxRun(kk) = ismember(runDataKeyValues(rr,kk),keyInfo{kk});
    end
    idx(rr) = all(idxRun);
end

runDataLim = runData(idx);

end

function outData = extractDataValues(runDataLim,infoStr)

% Returns the specific data from runDataLim corresponding to the string in
% infoStr.
% runData has already been limited only to the runs satisfying the key
% values.

if strcmpi(infoStr,'f')
    outData = [runDataLim.fillingDegree];
elseif strcmpi(infoStr,'w')
    outData = [runDataLim.rotationalSpeed_rpm];
elseif strcmpi(infoStr,'Fr')
    outData = [runDataLim.Froude];
elseif strcmpi(infoStr,'z')
    outData = [runDataLim.depth_cm];
elseif strcmpi(infoStr,'zL')
    outData = [runDataLim.relativeDepth];
elseif strcmpi(infoStr,'r')
    outData = [runDataLim.rotations];
elseif strcmpi(infoStr,'Mc')
    inter = [runDataLim.mixingIndex];
    outData = [inter.contactModified];
elseif strncmpi(infoStr,'Mv',2)
    inter = [runDataLim.mixingIndex];
    inter2 = [inter.variance];
    Ld = str2num(infoStr(3:end));
    idx = [inter2.L_over_d] == Ld;
    outData = [inter2(idx).M];
elseif strcmpi(infoStr,'M1M')
    inter = [runDataLim.mixingIndex];
    inter2 = [inter.firstOrder];
    outData = [inter2.M];
elseif strcmpi(infoStr,'M1i')
    inter = [runDataLim.mixingIndex];
    inter2 = [inter.firstOrder];
    outData = [inter2.M_inf];
elseif strcmpi(infoStr,'M1T')
    inter = [runDataLim.mixingIndex];
    inter2 = [inter.firstOrder];
    outData = [inter2.T_s];
elseif strcmpi(infoStr,'M2M')
    inter = [runDataLim.mixingIndex];
    inter2 = [inter.secondOrder];
    outData = [inter2.M];
elseif strcmpi(infoStr,'M2i')
    inter = [runDataLim.mixingIndex];
    inter2 = [inter.secondOrder];
    outData = [inter2.K];
elseif strcmpi(infoStr,'M2T')
    inter = [runDataLim.mixingIndex];
    inter2 = [inter.secondOrder];
    outData = [inter2.T_s];
elseif strcmpi(infoStr,'M2z')
    inter = [runDataLim.mixingIndex];
    inter2 = [inter.secondOrder];
    outData = [inter2.zeta];
elseif strcmpi(infoStr,'c')
    inter = [runDataLim.averageRedConcentration];
    outData = [inter.c];
elseif strcmpi(infoStr,'sAr')
    outData = [runDataLim.relativeSolidArea];
elseif strcmpi(infoStr,'ad')
    outData = [runDataLim.dynamicAngleOfRepose_degrees];
elseif strcmpi(infoStr,'am')
    outData = [runDataLim.medianAngle_degrees];
elseif strcmpi(infoStr,'as')
    outData = [runDataLim.stdAngleOfRepose_degrees];
elseif strcmpi(infoStr,'ax')
    outData = [runDataLim.maxAngleOfRepose_degrees];
else
    error(['Unknown variable: ' infoStr '.'])
end

end
