function CameraUFXCTProcessingDyn1(outputFolder)

% Collects all dynamic experimental (camera and UFXCT) runs that satisfy
% some given experimental conditions (filling degree, rotational speed,
% drum depth) and saves the calculated data for the frames when the drum is
% rotating in a common data.mat file in the output folder.
% All necessary parameters are read from the file parametersDynCU1.m, found
% in the output folder.

% Theodoros-Nestor Papapetrou
% Last edited: 17 Feb 2023

%% Input

% outputFolder: the path to the output folder, where all results are saved.
    % The output folder needs to exist and contain the file
    % 'parametersDynCU1.m' with the correct parameter values before calling
    % this function.
    % e.g. outputFolder = 'P:\Camera_ROFEX_data\f0p50';

%% Read parameters

run(fullfile(outputFolder,'parametersDynCU1.m'));

f = fillingDegree;
w = rotationalSpeed_rpm;
z = drumDepth_cm;

%% Find camera runs

% Check if zero is one of the specified depths
if ismembertol(0,z)
    
    % Read all files with camera runs
    sourceFilesCamera = dir(CameraSourceFolder);

    % Remove '.' and '..' from list
    idx = ismember({sourceFilesCamera.name},{'.','..'});
    sourceFilesCamera = sourceFilesCamera(~idx);
    noOfCameraFiles = numel(sourceFilesCamera);

    % Find all runs with given experimental conditions
    idx = true(1,noOfCameraFiles);
    for ii = 1:noOfCameraFiles
        try
            fileName = fullfile(CameraSourceFolder,...
                sourceFilesCamera(ii).name);
            load(fullfile(fileName,'data.mat'),'experimentalConditions');
            eC = experimentalConditions;
            if ~ismembertol(eC.fillingDegree,f)
                idx(ii) = false;
            elseif ~ismembertol(eC.rotationalSpeed_rpm,w)
                idx(ii) = false;
            end
        catch   % if any error occurs, e.g. there is no 'data.mat' file,
                % the corresponding file is discarded
            idx(ii) = false;
        end
    end
    sourceFilesCamera = sourceFilesCamera(idx);
    noOfCameraFiles = numel(sourceFilesCamera);
    
else % zero is not one of the specified depths, therefore no camera data
    noOfCameraFiles = 0;
end

%% Find UFXCT runs

% Check if any of the specified depths is not zero
if any(z,'all')
    
    % Read all files with UFXCT runs
    sourceFilesUFXCT = dir(UFXCTSourceFolder);

    % Remove '.' and '..' from list
    idx = ismember({sourceFilesUFXCT.name},{'.','..'});
    sourceFilesUFXCT = sourceFilesUFXCT(~idx);
    noOfUFXCTFiles = numel(sourceFilesUFXCT);

    % Find all runs with given experimental conditions
    idx = true(1,noOfUFXCTFiles);
    for ii = 1:noOfUFXCTFiles
        try
            fileName = fullfile(UFXCTSourceFolder,...
                sourceFilesUFXCT(ii).name);
            load(fullfile(fileName,'data.mat'),'experimentalConditions');
            eC = experimentalConditions;
            if ~ismembertol(eC.fillingDegree,f)
                idx(ii) = false;
            elseif ~ismembertol(eC.rotationalSpeed_rpm,w)
                idx(ii) = false;
            elseif ~ismembertol(eC.depth_cm,z)
                idx(ii) = false;
            end
        catch   % if any error occurs, e.g. there is no 'data.mat' file,
                % the corresponding file is discarded
            idx(ii) = false;
        end
    end
    sourceFilesUFXCT = sourceFilesUFXCT(idx);
    noOfUFXCTFiles = numel(sourceFilesUFXCT);
    
else % no non-zero depths specified
    noOfUFXCTFiles = 0;
end

%% Calculate constants

clear f w z
constants.drumDiameter_cm = drumDiameter_cm;
constants.drumLength_cm = drumLength_cm;
constants.particleDiameter_mm = particleDiameter_mm;
drumRadius_m = drumDiameter_cm / 200;
g_SI = 9.81;

%% Collect UFXCT data

L_over_d = []; % to be used for the variance method with camera data
if noOfUFXCTFiles > 0
    for ii = noOfUFXCTFiles:-1:1
        
        % Load data
        jj = noOfCameraFiles + ii;
        fileName = fullfile(UFXCTSourceFolder,sourceFilesUFXCT(ii).name);
        data = load(fullfile(fileName,'data.mat'));
        eC = data.experimentalConditions;
        rF = data.calculations.rotationFrames;
        
        % Save experimental conditions
        runData(jj).fillingDegree = eC.fillingDegree;
        
        w = eC.rotationalSpeed_rpm;
        runData(jj).rotationalSpeed_rpm = w;
        rotationalSpeed_SI = w*pi/30;
        Froude = (rotationalSpeed_SI)^2*drumRadius_m/g_SI;
        runData(jj).Froude = Froude;
        
        z = eC.depth_cm;
        runData(jj).depth_cm = z;
        runData(jj).relativeDepth = z / drumLength_cm;
        
        % Save calculations
        runData(jj).rotations = rF.rotations;
        
        mixingIndex = rF.mixingIndex;
        runData(jj).mixingIndex = mixingIndex;
        Ld = [mixingIndex.variance.L_over_d];
        L_over_d = union(L_over_d,Ld);
        clear mixingIndex
        
        c = rF.averageRedConcentration;
        runData(jj).averageRedConcentration = c;
        
        fc = rF.fillingDegree;
        runData(jj).fillingDegreeCalc = fc;
        
        aor = rF.dynamicAngleOfRepose_degrees;
        runData(jj).dynamicAngleOfRepose_degrees = aor;
        
        maor = rF.medianAngle_degrees;
        runData(jj).medianAngle_degrees = maor;
        
        stdaor = rF.stdAngleOfRepose_degrees;
        runData(jj).stdAngleOfRepose_degrees = stdaor;
    end
end

%% Collect camera data, adjust format to UFXCT data

if noOfCameraFiles > 0
    for ii = noOfCameraFiles:-1:1
        
        % Load data
        fileName = fullfile(CameraSourceFolder,sourceFilesCamera(ii).name);
        load(fullfile(fileName,'data.mat'),'calculations',...
            'experimentalConditions','rotationsExact',...
            'startingFrameNumber');
        eC = experimentalConditions;
        
        % Save experimental conditions
        runData(ii).fillingDegree = eC.fillingDegree;
        
        w = eC.rotationalSpeed_rpm;
        runData(ii).rotationalSpeed_rpm = w;
        rotationalSpeed_SI = w*pi/30;
        Froude = (rotationalSpeed_SI)^2*drumRadius_m/g_SI;
        runData(ii).Froude = Froude;
        
        runData(ii).depth_cm = 0;
        runData(ii).relativeDepth = 0;
        
        % Save calculations
        firstFrame = startingFrameNumber;
        rotations = rotationsExact;
        lastFrame = find(rotations == max(rotations),1);
        frameRange = firstFrame:lastFrame;
        rotations = rotations(frameRange);
        runData(ii).rotations = rotations;
        
        Mc = calculations.modifiedContactMixingIndex(frameRange);
        mixingIndex.contactModified = Mc';
        if ~isempty(L_over_d) % variance method
            seqPvidGran = calculations.Pvid(:,:,frameRange);
            d_p = calculations.particleDiameter_pixels;
            for jj = numel(L_over_d):-1:1
                Ld = L_over_d(jj);
                Mv = M_variance3D(seqPvidGran,d_p,Ld);
                mixingIndex.variance(jj).L_over_d = Ld;
                mixingIndex.variance(jj).M = Mv';
            end
        end
        runData(ii).mixingIndex = mixingIndex;
        clear mixingIndex
        
        c = calculations.averageRedConcentration(frameRange);
        runData(ii).averageRedConcentration.c = c';
        
        tSL = calculations.topSurfaceLinear;
        fc = tSL.fillingDegree(frameRange);
        runData(ii).fillingDegreeCalc = fc;
        
        aor = tSL.angleOfRepose_degrees(frameRange);
        runData(ii).dynamicAngleOfRepose_degrees = aor;
        
        runData(ii).medianAngle_degrees = median(aor,'omitnan');
        runData(ii).stdAngleOfRepose_degrees = std(aor,'omitnan');
        
    end
end

%% Save results to data.mat

data = fullfile(outputFolder,'data.mat');
if isfile(data)
    save(data,'constants','runData','-append');
else
    save(data,'constants','runData');
end
