function CameraUFXCTProcessingDyn2(outputFolder)

% Loads the calculated data for all dynamic runs (variable 'runData' in
% data.mat in the source folder) and fits a first- and a second-order
% dynamics curve to the modified contact mixing index as a function of
% rotations for each run. The parameters of each of the two curves are
% fitted using nonlinear least squares regression. For camera runs, the
% initial conditions are taken to be zero. For UFXCT runs, the initial
% conditions are fitted from the data. The values of the mixing index as a
% function of the number of rotations are calculated for both curves
% (first- and second-order).
% A first-order dynamics curve is fit to the average red concentration as a
% function of rotations for each camera run. For all camera runs, the
% initial conditions are fitted from the data.
% The maximum (by absolute value) angle of repose
% ('maxAngleOfRepose_degrees') is also calculated.
% All necessary parameters are read from the file parametersDynCU2.m, found
% in the output folder.
% The file data.mat is copied from the source folder to the output
% folder. All results are appended in the variable 'runData' in data.mat in
% the output folder.

% Theodoros-Nestor Papapetrou
% Last edited: 23 Feb 2023

%% Input

% outputFolder: the path to the output folder, where all results are saved.
    % The output folder needs to exist and contain the file
    % 'parametersDynCU2.m' with the correct parameter values before calling
    % this function.
    % e.g. outputFolder = 'P:\Camera_ROFEX_data\f0p50';

%% Read parameters

run(fullfile(outputFolder,'parametersDynCU2.m'));

%% Copy data.mat to output folder, load data

if ~isequal(sourceFolder,outputFolder)
    data = fullfile(sourceFolder,'data.mat');
    copyfile(data,outputFolder)
end

data = fullfile(outputFolder,'data.mat');
load(data,'runData');
noOfRuns = numel(runData);

%% Calculations for each run

for ii = 1:noOfRuns
    disp(['Run ' num2str(ii) '/' num2str(noOfRuns) '...'])
    
    %% Mixing index - dynamic fit
    
    % Data
    xdata = runData(ii).rotations';
    mixingIndex = runData(ii).mixingIndex;
    ydata = mixingIndex.contactModified';
    
    idxval = ~isnan(ydata); % indices of non-NaN y-values
    yVal = ydata(idxval); % non-NaN y-values
    xVal = xdata(idxval); % x-values correxponding to non-NaN y-values
    clear firstOrder
    
    if runData(ii).depth_cm == 0 % camera
        
        % 1st order fit - zero initial conditions
        parameters = modelParameters(xVal,yVal,0);
        T = parameters(1);
        M_inf = parameters(2);
        M = M_inf - M_inf*exp(-xdata'/T);
        firstOrder.T_rotations = T;
        firstOrder.M_inf = M_inf;
        firstOrder.M_0 = 0;
        firstOrder.M = M;
        mixingIndex.firstOrder = firstOrder;
        
        % 2nd order fit - zero initial conditions
        parameters = modelParameters(xVal,yVal,2);
        T = parameters(1);
        z = parameters(2);
        K = parameters(3);
        t_w = [xdata xdata];
        t_w(1,2) = 0;
        t_w(2:end,2) = 1;
        M = solve2ndorderLinDE(T,z,K,t_w,xdata,[0;0])';
        M_max = max(M);
        t_p = xdata(find(M == M_max,1));
        secondOrder.T_rotations = T;
        secondOrder.zeta = z;
        secondOrder.K = K;
        secondOrder.M_0 = 0;
        secondOrder.dM_0 = 0;
        secondOrder.t_p_rotations = t_p;
        secondOrder.M_max = M_max;
        secondOrder.M = M;
        mixingIndex.secondOrder = secondOrder;
        
    else % UFXCT
        
        % 1st order fit - estimated initial conditions
        parameters = modelParameters(xVal,yVal,1);
        T = parameters(1);
        M_inf = parameters(2);
        M_0 = parameters(3);
        M = M_inf + (M_0 - M_inf)*exp(-xdata'/T);
        firstOrder.T_rotations = T;
        firstOrder.M_inf = M_inf;
        firstOrder.M_0 = M_0;
        firstOrder.M = M;
        mixingIndex.firstOrder = firstOrder;

        % 2nd order fit - estimated initial conditions
        parameters = modelParameters(xVal,yVal,3);
        T = parameters(1);
        z = parameters(2);
        K = parameters(3);
        M_0 = parameters(4);
        dM_0 = parameters(5);
        t_w = [xdata xdata];
        t_w(1,2) = 0;
        t_w(2:end,2) = 1;
        M = solve2ndorderLinDE(T,z,K,t_w,xdata,[M_0;dM_0])';
        M_max = max(M);
        t_p = xdata(find(M == M_max,1));
        secondOrder.T_rotations = T;
        secondOrder.zeta = z;
        secondOrder.K = K;
        secondOrder.M_0 = M_0;
        secondOrder.dM_0 = dM_0;
        secondOrder.t_p_rotations = t_p;
        secondOrder.M_max = M_max;
        secondOrder.M = M;
        mixingIndex.secondOrder = secondOrder;
        
    end
    
    runData(ii).mixingIndex = mixingIndex;
    
    %% Average red concentration - dynamic fit (manual - camera only)
    
    if runData(ii).depth_cm == 0
        
        % Data
        averageRedConcentration = runData(ii).averageRedConcentration;
        ydata = averageRedConcentration.c';
        yVal = ydata(idxval); % non-NaN y-values
    
        c_inf = mean(yVal(xVal > 5));
        c_0_guess = yVal(1);
        T_guess = 2; % rotations
        fun = @(b,x)c_inf+(b(1)-c_inf)*exp(-x/b(2));
        beta0 = [c_0_guess T_guess];
        % opt = statset('MaxIter',2000);
        % mdl = fitnlm(xVal,yVal,fun,beta0,'options',opt);
        mdl = fitnlm(xVal,yVal,fun,beta0);
        beta = mdl.Coefficients.Estimate;
    
        clear firstOrder
        firstOrder.T_rotations = beta(2);
        firstOrder.c_inf = c_inf;
        firstOrder.c_0 = beta(1);
        firstOrder.c = fun(beta,xdata);
        averageRedConcentration.firstOrder = firstOrder;
        runData(ii).averageRedConcentration = averageRedConcentration;
        
    end
    
    %% Maximum angle of repose
    amax = max(abs(runData(ii).dynamicAngleOfRepose_degrees));
    runData(ii).maxAngleOfRepose_degrees = amax;
    
end

%% Save results to data.mat

data = fullfile(outputFolder,'data.mat');
if isfile(data)
    save(data,'runData','-append');
else
    save(data,'runData');
end
