function CameraUFXCTProcessingDyn3(outputFolder)

% Groups the calculated and fitted parameters of all runs by Froude number
% and by drum depth in two new fields of data.mat. A curve of the form
%   b1 + b2*(exp(-z/b3) + exp((L-z)/b3))    (1)
% is fitted for the steady-state angle as a function of depth, for each
% Froude number separately. A curve of the form (1) is also manually fitted
% for the steady-state concentration as a function of depth, for all Froude
% numbers.
% All necessary parameters are read from the file parametersDynCU3.m, found
% in the output folder.
% The file data.mat is copied from the source folder to the output
% folder. All results are appended in the variables 'byFroudeExp',
% 'byFroudeFit', 'fitVSdepth', 'byDepthExp', 'byDepthFit', 'fitVSFroude'
% and 'totalFits' in data.mat in the output folder.

% Theodoros-Nestor Papapetrou
% Last edited: 24 Feb 2023

%% Input

% outputFolder: the path to the output folder, where all results are saved.
    % The output folder needs to exist and contain the file
    % 'parametersDynCU3.m' with the correct parameter values before calling
    % this function.
    % e.g. outputFolder = 'P:\Camera_ROFEX_data\f0p50';

%% Read parameters

run(fullfile(outputFolder,'parametersDynCU3.m'));

%% Copy data.mat to output folder, load data

if ~isequal(sourceFolder,outputFolder)
    data = fullfile(sourceFolder,'data.mat');
    copyfile(data,outputFolder)
end

data = fullfile(outputFolder,'data.mat');
load(data,'constants','runData');
noOfRuns = numel(runData);
L = constants.drumLength_cm;

%% Group data by Froude number in new field

jj = 0; % no of different Froude numbers
w = []; % rotational speed (rpm)
for ii = 1:noOfRuns
    if ~ismember(runData(ii).rotationalSpeed_rpm,w)
        jj = jj + 1;
        w = [w runData(ii).rotationalSpeed_rpm];
        byFroudeExp(1,jj).rotationalSpeed_rpm = w(end);
        byFroudeFit(1,jj).rotationalSpeed_rpm = w(end);
        byFroudeExp(jj).Froude = runData(ii).Froude;
        byFroudeFit(jj).Froude = runData(ii).Froude;
    end
end

noOfFroude = numel(w);
byFroudeExp = table2struct(sortrows(struct2table(byFroudeExp), 'Froude'))';
byFroudeFit = table2struct(sortrows(struct2table(byFroudeFit), 'Froude'))';
noOfDepths = zeros(1,noOfFroude); % no of drum depths for each Fr

for jj = 1:noOfFroude
    z = []; % drum depth (cm)
    zL = []; % relative drum depth
    for ii = 1:noOfRuns
        if runData(ii).rotationalSpeed_rpm == w(jj)
            noOfDepths(jj) = noOfDepths(jj) + 1;
            z = [z runData(ii).depth_cm];
            zL = [zL runData(ii).relativeDepth];
        end
    end
    byFroudeExp(jj).depth_cm = z;
    byFroudeExp(jj).relativeDepth = zL;
    zmax = max(z); zmin = min(z);
    byFroudeFit(jj).depth_cm = linspace(zmin,zmax,1000);
    zLmax = max(zL); zLmin = min(zL);
    byFroudeFit(jj).relativeDepth = linspace(zLmin,zLmax,1000);
end

z = [byFroudeExp(:).depth_cm];
zmax = max(z); zmin = min(z);
fitVSdepth.depth_cm = linspace(zmin,zmax,1000);
fitVSdepth.relativeDepth = linspace(zmin,zmax,1000)/L;

for jj = 1:noOfFroude
    M_0 = zeros(1,noOfDepths(jj));
    M_inf = zeros(1,noOfDepths(jj));
    tau1 = zeros(1,noOfDepths(jj));
    tau2 = zeros(1,noOfDepths(jj));
    zeta = zeros(1,noOfDepths(jj));
    t_p = zeros(1,noOfDepths(jj));
    M_max = zeros(1,noOfDepths(jj));
    c_0 = zeros(1,noOfDepths(jj));
    c_inf = zeros(1,noOfDepths(jj));
    tau_c = zeros(1,noOfDepths(jj));
    a_med = zeros(1,noOfDepths(jj));
    a_max = zeros(1,noOfDepths(jj));
    
    kk = 0;
    for ii = 1:noOfRuns
        if runData(ii).rotationalSpeed_rpm == w(jj)
            kk = kk + 1;
            M_0(kk) = runData(ii).mixingIndex.firstOrder.M_0;
            M_inf(kk) = runData(ii).mixingIndex.firstOrder.M_inf;
            tau1(kk) = runData(ii).mixingIndex.firstOrder.T_rotations;
            tau2(kk) = runData(ii).mixingIndex.secondOrder.T_rotations;
            zeta(kk) = runData(ii).mixingIndex.secondOrder.zeta;
            t_p(kk) = runData(ii).mixingIndex.secondOrder.t_p_rotations;
            M_max(kk) = runData(ii).mixingIndex.secondOrder.M_max;
            c_0(kk) = runData(ii).averageRedConcentration.firstOrder.c_0;
            c_inf(kk) = ...
                runData(ii).averageRedConcentration.firstOrder.c_inf;
            tau_c(kk) = ...
                runData(ii).averageRedConcentration.firstOrder.T_rotations;
            a_med(kk) = runData(ii).medianAngle_degrees;
            a_max(kk) = runData(ii).maxAngleOfRepose_degrees;
        end
    end
    
    byFroudeExp(jj).M_0 = M_0;
    byFroudeExp(jj).M_inf = M_inf;
    byFroudeExp(jj).tau1_rotations = tau1;
    byFroudeExp(jj).tau2_rotations = tau2;
    byFroudeExp(jj).zeta = zeta;
    byFroudeExp(jj).t_p_rotations = t_p;
    byFroudeExp(jj).M_max = M_max;
    byFroudeExp(jj).c_0 = c_0;
    byFroudeExp(jj).c_inf = c_inf;
    byFroudeExp(jj).tau_c_rotations = tau_c;
    byFroudeExp(jj).a_med_degrees = a_med;
    byFroudeExp(jj).a_max_degrees = a_max; 
    
end

%% Group data by drum depth in new field

jj = 0; % no of different drum depths
z = []; % drum depth (cm)
for ii = 1:noOfRuns
    if ~ismember(runData(ii).depth_cm,z)
        jj = jj + 1;
        z = [z runData(ii).depth_cm];
        byDepthExp(1,jj).depth_cm = z(end);
        byDepthFit(1,jj).depth_cm = z(end);
        byDepthExp(jj).relativeDepth = runData(ii).relativeDepth;
        byDepthFit(jj).relativeDepth = runData(ii).relativeDepth;
    end
end

noOfDepth = numel(z);
byDepthExp = table2struct(sortrows(struct2table(byDepthExp),'depth_cm'))';
byDepthFit = table2struct(sortrows(struct2table(byDepthFit),'depth_cm'))';
noOfFroudes = zeros(1,noOfDepth); % no of drum depths for each Fr

for jj = 1:noOfDepth
    w = []; % rotational speed (rpm)
    Fr = []; %
    for ii = 1:noOfRuns
        if runData(ii).depth_cm == z(jj)
            noOfFroudes(jj) = noOfFroudes(jj) + 1;
            w = [w runData(ii).rotationalSpeed_rpm];
            Fr = [Fr runData(ii).Froude];
        end
    end
    byDepthExp(jj).rotationalSpeed_rpm = w;
    byDepthExp(jj).Froude = Fr;
    wmax = max(w); wmin = min(w);
    byDepthFit(jj).rotationalSpeed_rpm = linspace(wmin,wmax,1000);
    Frmax = max(Fr); Frmin = min(Fr);
    byDepthFit(jj).Froude = linspace(Frmin,Frmax,1000);
end

w = [byDepthExp(:).rotationalSpeed_rpm];
Fr = [byDepthExp(:).Froude];
wmax = max(w); wmin = min(w);
fitVSFroude.rotationalSpeed_rpm = linspace(wmin,wmax,1000);
Frmax = max(Fr); Frmin = min(Fr);
fitVSFroude.Froude = linspace(Frmin,Frmax,1000);

for jj = 1:noOfDepth
    M_0 = zeros(1,noOfFroudes(jj));
    M_inf = zeros(1,noOfFroudes(jj));
    tau1 = zeros(1,noOfFroudes(jj));
    tau2 = zeros(1,noOfFroudes(jj));
    zeta = zeros(1,noOfFroudes(jj));
    t_p = zeros(1,noOfFroudes(jj));
    M_max = zeros(1,noOfFroudes(jj));
    c_0 = zeros(1,noOfFroudes(jj));
    c_inf = zeros(1,noOfFroudes(jj));
    tau_c = zeros(1,noOfFroudes(jj));
    a_med = zeros(1,noOfFroudes(jj));
    a_max = zeros(1,noOfFroudes(jj));
    
    kk = 0;
    for ii = 1:noOfRuns
        if runData(ii).depth_cm == z(jj)
            kk = kk + 1;
            M_0(kk) = runData(ii).mixingIndex.firstOrder.M_0;
            M_inf(kk) = runData(ii).mixingIndex.firstOrder.M_inf;
            tau1(kk) = runData(ii).mixingIndex.firstOrder.T_rotations;
            tau2(kk) = runData(ii).mixingIndex.secondOrder.T_rotations;
            zeta(kk) = runData(ii).mixingIndex.secondOrder.zeta;
            t_p(kk) = runData(ii).mixingIndex.secondOrder.t_p_rotations;
            M_max(kk) = runData(ii).mixingIndex.secondOrder.M_max;
            c_0(kk) = runData(ii).averageRedConcentration.firstOrder.c_0;
            c_inf(kk) = ...
                runData(ii).averageRedConcentration.firstOrder.c_inf;
            tau_c(kk) = ...
                runData(ii).averageRedConcentration.firstOrder.T_rotations;
            a_med(kk) = runData(ii).medianAngle_degrees;
            a_max(kk) = runData(ii).maxAngleOfRepose_degrees;
        end
    end
    
    byDepthExp(jj).M_0 = M_0;
    byDepthExp(jj).M_inf = M_inf;
    byDepthExp(jj).tau1_rotations = tau1;
    byDepthExp(jj).tau2_rotations = tau2;
    byDepthExp(jj).zeta = zeta;
    byDepthExp(jj).t_p_rotations = t_p;
    byDepthExp(jj).M_max = M_max;
    byDepthExp(jj).c_0 = c_0;
    byDepthExp(jj).c_inf = c_inf;
    byDepthExp(jj).tau_c_rotations = tau_c;
    byDepthExp(jj).a_med_degrees = a_med;
    byDepthExp(jj).a_max_degrees = a_max; 
    
end

%% Fit median angles to curve of Dury et al. for each Froude number

for jj = 1:noOfFroude
    disp([num2str(jj) '/' num2str(noOfFroude)])

    z = byFroudeExp(jj).depth_cm;
    z_all = byFroudeFit(jj).depth_cm;
    
    a_med = byFroudeExp(jj).a_med_degrees;
    a_bulk_guess = min(abs(a_med));
    a_wall_guess = max(abs(a_med));
    z_a_guess = 2; % cm
    fun = @(b,x)b(1)+(b(2)-b(1))*(exp(-x/b(3))+exp(-(L-x)/b(3)));
    beta0 = [a_bulk_guess a_wall_guess z_a_guess];
    opt = statset('MaxIter',2000);
    mdl = fitnlm(z,a_med,fun,beta0,'options',opt); % preliminary fit
    beta = mdl.Coefficients.Estimate;
    a_all = fun(beta,z_all); % steady state angle - fitted
    a_bulk = a_all(end); % steady state angle in the bulk
    a_wall = a_all(1); % steady state angle at the wall
    delta_a = a_wall - a_bulk; % steady state angle difference
    funCor = @(b,x)a_bulk+delta_a*(exp(-x/b)+exp(-(L-x)/b));
    mdl = fitnlm(z,a_med,funCor,z_a_guess,'options',opt); % corrected fit
    z_a = mdl.Coefficients.Estimate; % angle wall effect length
    
    byFroudeExp(jj).a_bulk_degrees = a_bulk;
    byFroudeExp(jj).a_wall_degrees = a_wall;
    byFroudeExp(jj).z_a_cm = z_a;
    byFroudeExp(jj).z_a_relative = z_a/L;
    byFroudeFit(jj).a_med_degrees = a_all;
       
end

%% Fit steady state concentration to one single curve of Dury et al.

% z = [byFroudeExp(:).depth_cm];
% z_all = fitVSdepth.depth_cm;
% c_inf = [byFroudeExp(:).c_inf];
% % c_bulk_guess = min(abs(c_inf));
% % c_wall_guess = max(abs(c_inf));
% % z_c_guess = 2; % cm
% c_bulk_guess = .33;
% c_wall_guess = .77;
% z_c_guess = 4.3/100*L; % cm
% fun = @(b,x)b(1)+(b(2)-b(1))*(exp(-x/b(3))+exp(-(L-x)/b(3)));
% beta0 = [c_bulk_guess c_wall_guess z_c_guess];
% opt = statset('MaxIter',2000);
% mdl = fitnlm(z,c_inf,fun,beta0,'options',opt);
% beta = mdl.Coefficients.Estimate;
% c_all = fun(beta,z_all); % steady state concentration - fitted
% c_bulk = beta(1); % steady state concentration in the bulk
% c_wall = beta(2); % steady state concentration at the wall
% z_c = beta(3); % concentration wall effect length
% 
% fitVSdepth.c_inf = c_all;
% c_inf_vs_depth.c_bulk = c_bulk;
% c_inf_vs_depth.c_wall = c_wall;
% c_inf_vs_depth.z_c_cm = z_c;
% c_inf_vs_depth.z_c_relative = z_c/L;
% totalFits.c_inf_vs_depth = c_inf_vs_depth;

%% Manual fit of steady state concentration to one curve of Dury et al.

z = [byDepthExp(:).depth_cm];
z_all = fitVSdepth.depth_cm;
c_bulk = mean([byDepthExp(z>0).c_inf]);
c_wall = mean([byDepthExp(z==0).c_inf]);
z_c = steadyConcentrationCharDepth_cm;
c_all = c_bulk+(c_wall-c_bulk)*(exp(-z_all/z_c)+exp(-(L-z_all)/z_c));

fitVSdepth.c_inf = c_all;
c_inf_vs_depth.c_bulk = c_bulk;
c_inf_vs_depth.c_wall = c_wall;
c_inf_vs_depth.z_c_cm = z_c;
c_inf_vs_depth.z_c_relative = z_c/L;
totalFits.c_inf_vs_depth = c_inf_vs_depth;

%% Save results to data.mat

data = fullfile(outputFolder,'data.mat');
if isfile(data)
    save(data,'byFroudeExp','byFroudeFit','byDepthExp','byDepthFit',...
        'totalFits','fitVSdepth','fitVSFroude','-append');
else
    save(data,'byFroudeExp','byFroudeFit','byDepthExp','byDepthFit',...
        'totalFits','fitVSdepth','fitVSFroude');
end
