function UFXCTProcessingDyn4(outputFolder)

% Merges the calculated results (mixing index, average red concentration,
% filling degree and dynamic angle of repose) of all parts of one dynamic
% ROFEX run. From each part, only the data from frames when the drum
% rotates are concatenated. The drift of the average red concentration is
% corrected and the median dynamic angle of repose is calculated.
% All necessary parameters are read from the file parametersDyn4.m, found
% in the output folder.
% The experimental conditions and the merged calculated quantities are
% saved in the file data.mat in the output folder
% (data.calculations.rotationFrames).
% A plot of the modified contact method mixing index ('M'), a plot of the
% average red concentration ('c'), a plot of the filling degree ('f') and a
% plot of the dynamic angle of repose ('a'), as a function of rotations,
% are also saved in the output folder.

% Theodoros-Nestor Papapetrou
% Last edited: 15 Jun 2023

%% Input

% outputFolder: the path to the output folder, where all results are saved.
    % The output folder needs to exist and contain the file
    % 'parametersDyn4.m' with the correct parameter values before calling
    % this function.
    % e.g. outputFolder = 'P:\ROFEX data\202103\run1';

%% Read parameters

run(fullfile(outputFolder,'parametersDyn4.m'));

%% Load data from all parts of given run

pattern = ['run' num2str(experimentalRun) '_part*'];
sourceFiles = dir(fullfile(sourceFolder,pattern));
noOfParts = numel(sourceFiles);

for ii = noOfParts:-1:1
    partFolder = fullfile(sourceFiles(ii).folder,sourceFiles(ii).name);
    partData = fullfile(partFolder,'data.mat');
    data(ii) = load(partData,'experimentalConditions',...
        'characteristicFrames','calculations');
end

%% Check if parts are distinct

for ii = 1:noOfParts
    data(ii).part = data(ii).experimentalConditions.part;
end
part = [data.part];
if numel(part) > numel(unique(part))
    error('Some part is read more than once.')
end

%% Sort data by part

data = table2struct(sortrows(struct2table(data), 'part'));

%% Check if experimental conditions are equal for all parts

expConditions = [data.experimentalConditions];
expConditions = rmfield(expConditions,'part');
expConditions1 = expConditions(1);
if noOfParts > 1
    for ii = 2:noOfParts
        if ~isequal(expConditions1,expConditions(ii))
            error(['Different experimental conditions in part '...
                num2str(ii) '.'])
        end
    end
end
expConditions1.rotationsPerPart = expConditions1.noOfRotations;
expConditions1 = rmfield(expConditions1,'noOfRotations');
expConditions1.noOfParts = noOfParts;
experimentalConditions = expConditions1;

%% Find rotation frames

% Number of rotation frames
charFrames = [data.characteristicFrames];
rotStart = [charFrames.rotationStart];
rotEnd = [charFrames.rotationEnd];
noOfRotationFrames = sum(rotEnd) - sum(rotStart) + 1;

% Input frame range in each part (the range of rotation frames)
inRange = cell(1,noOfParts);
inRange{1} = rotStart(1):rotEnd(1);
if noOfParts > 1
    for ii = 2:noOfParts
        inRange{ii} = (rotStart(ii) + 1):rotEnd(ii);
    end
end

% Output frame range for each part in the merged data
outRange = cell(1,noOfParts);
lastFrame = 0;
for ii = 1:noOfParts
    firstFrame = lastFrame + 1;
    lastFrame = lastFrame + numel(inRange{ii});
    outRange{ii} = firstFrame:lastFrame;
end

%% Rotation and concentration values of each part

% The rotation (rot) and (uncorrected) concentration (c) values from the
% input range of each part are collected. A line of the form
% c = s0*rot + b0 
% is fitted for each part. The average of rot and c for each part, as well
% as the global average of c for the entire run, are calculated. The global
% rotation values are also calculated and concatenated.

calcsPart = [data.calculations];
rotationsPart = cell(1,noOfParts);
cPartUncorrected = cell(1,noOfParts);
s0 = zeros(1,noOfParts); % slope of c-rot fitted line
b0 = zeros(1,noOfParts); % intercept of c-rot fitted line
cAveragePart = zeros(1,noOfParts); 
rotAveragePart = zeros(1,noOfParts);
rotations = zeros(1,noOfRotationFrames);
rotationsDone = 0;
cGlobUncorrected = zeros(1,noOfRotationFrames);
for ii = 1:noOfParts
    rotationsPart{ii} = calcsPart(ii).rotations(inRange{ii});
    rotations(outRange{ii}) = rotationsPart{ii} + rotationsDone;
    rotationsDone = rotationsDone + expConditions1.rotationsPerPart;
    cPartUncorrected{ii} = calcsPart(ii).averageRedConcentration(inRange{ii});
    idxval = ~isnan(cPartUncorrected{ii});
    cVal = cPartUncorrected{ii}(idxval);
    rotVal = rotationsPart{ii}(idxval);
    linfit = polyfit(rotVal,cVal,1);
    s0(ii) = linfit(1);
    b0(ii) = linfit(2);
    cAveragePart(ii) = mean(cVal);
    rotAveragePart(ii) = mean(rotVal);
    cGlobUncorrected(outRange{ii}) = cPartUncorrected{ii};
end
cAverageGlob = mean(cGlobUncorrected,'omitnan');

%% Concentration correction

% The concentration data of each part are corrected so that the corrected
% concentration and rotation values fit the line
% c = s*rot + b,
% where s = 0 and the average concentration value for each part is equal to
% the global average of the uncorrected values (cAverageGlob).

s = 0;
cGlobCorrected = zeros(1,noOfRotationFrames);
for ii = 1:noOfParts
    rot = rotationsPart{ii};
    cUncor = cPartUncorrected{ii};
    cCor = cUncor + (s-s0(ii))*(rot-rotAveragePart(ii))...
        + cAverageGlob - cAveragePart(ii);
    cGlobCorrected(outRange{ii}) = cCor;
end

%% Save results to 'rotationFrames' struct

MI = [calcsPart.mixingIndex];
L_over_d = [MI(1).variance.L_over_d];
Mc = zeros(1,noOfRotationFrames); % mixing index - modified contact method
f = zeros(1,noOfRotationFrames); % filling degree
aor = zeros(1,noOfRotationFrames); % dynamic angle of repose
for ii = 1:noOfParts
    Mc(outRange{ii}) = MI(ii).contactModified(inRange{ii});
    f(outRange{ii}) = calcsPart(ii).fillingDegree(inRange{ii});
    aor(outRange{ii}) = ...
        calcsPart(ii).dynamicAngleOfRepose_degrees(inRange{ii});
end

rotationFrames.rotations = rotations;
mixingIndex.contactModified = Mc;

% Mixing index - variance method
if ~isempty(L_over_d)
     for jj = numel(L_over_d):-1:1
        Ld = L_over_d(jj);
        Mv = zeros(1,noOfRotationFrames);
        for ii = noOfParts:-1:1
            kk = [MI(ii).variance.L_over_d] == Ld;
            Mv(outRange{ii}) = MI(ii).variance(kk).M(inRange{ii});
        end
        Mv(1) = MI(1).variance(kk).M(rotStart(1));
        mixingIndex.variance(jj).L_over_d = Ld;
        mixingIndex.variance(jj).M = Mv;
    end
end

rotationFrames.mixingIndex = mixingIndex;
rotationFrames.averageRedConcentration = cGlobCorrected;
rotationFrames.fillingDegree = f;
rotationFrames.dynamicAngleOfRepose_degrees = aor;

% Median dynamic angle of repose
rotationFrames.medianAngle_degrees = median(aor,'omitnan');
rotationFrames.stdAngleOfRepose_degrees = std(aor,'omitnan');

calculations.rotationFrames = rotationFrames;

%% Figures

% Plot mixing index
figure;
plot(rotations,Mc);
xlabel('rotations')
ylabel('M');
saveas(gcf,fullfile(outputFolder,'M_rotationFrames.png'));
close

% Plot average red concentration
figure;
plot(rotations,cGlobCorrected);
xlabel('rotations')
ylabel('c');
saveas(gcf,fullfile(outputFolder,'c_rotationFrames.png'));
close

% Plot filling degree
figure;
plot(rotations,f);
xlabel('rotations')
ylabel('f');
saveas(gcf,fullfile(outputFolder,'f_rotationFrames.png'));
close

% Plot dynamic angle of repose
figure;
plot(rotations,aor);
xlabel('rotations')
ylabel('a (degrees)');
saveas(gcf,fullfile(outputFolder,'a_rotationFrames.png'));
close

%% Save results in data.mat

data = fullfile(outputFolder,'data.mat');
if isfile(data)
    save(data,'experimentalConditions','calculations','-append');
else
    save(data,'experimentalConditions','calculations');
end
