function UFXCTProcessingDyn5(outputFolder)

% Calculates the parameters for the manual fitting of a first-order dynamic
% curve to the experimental values of average red concentration as a
% function of rotations, for the rotating frames only.
% All necessary parameters are read from the file parametersDyn5.m, found
% in the output folder.
% The calculated parameters and the fitted values of average red
% concentration are saved in the field 'firstOrder' of
% 'averageRedConcentration' of 'rotationFrames' of 'calculations', in the
% file data.mat in the output folder. The experimental values are moved to
% the field 'c' of 'averageRedConcentration', in the same location.
% A plot of the fitted concentration values, on top of the experimental
% ones ('c_fit'), is also saved in the output folder.

% Theodoros-Nestor Papapetrou
% Last edited: 23 Feb 2023

%% Input

% outputFolder: the path to the output folder, where all results are saved.
    % The output folder needs to exist and contain the file
    % 'parametersDyn5.m' with the correct parameter values before calling
    % this function.
    % e.g. outputFolder = 'P:\ROFEX data\202103\run1';

%% Read parameters

run(fullfile(outputFolder,'parametersDyn5.m'));
    
%% Copy data.mat to output folder, load data

if ~isequal(sourceFolder,outputFolder)
    data = fullfile(sourceFolder,'data.mat');
    copyfile(data,outputFolder)
end

data = fullfile(outputFolder,'data.mat');
load(data,'calculations');

%% Calculations

% Rotation values
rotationFrames = calculations.rotationFrames;
rot = rotationFrames.rotations;

% Experimental concentration values
c = rotationFrames.averageRedConcentration;

% Indices of steady state
steadyRangeMid = mean(steadyRange(1:2));
steadyRangeRadius = diff(steadyRange(1:2))/2;
idx = abs(rot - steadyRangeMid) <= steadyRangeRadius;

c_inf = mean(c(idx),'omitnan');
c_calc = c_inf + (c_0 - c_inf)*exp(-rot/tau_c);

firstOrder.T_rotations = tau_c;
firstOrder.c_inf = c_inf;
firstOrder.c_0 = c_0;
firstOrder.c = c_calc;

averageRedConcentration.c = c;
averageRedConcentration.firstOrder = firstOrder;
rotationFrames.averageRedConcentration = averageRedConcentration;
calculations.rotationFrames = rotationFrames;

%% Plot

figure
hold on
plot(rot,c);
plot(rot,c_calc);
xlim([0 30])
xlabel('rotations')
ylabel('c');
legend({'exp','fit'});
saveas(gcf,fullfile(outputFolder,'c_fit.png'));
close

%% Save results in data.mat

data = fullfile(outputFolder,'data.mat');
if isfile(data)
    save(data,'calculations','-append');
else
    save(data,'calculations');
end
