function seq = UFXCTProcessingFul0(outputFolder)

% Loads all ROFEX image sequences (tiff files) that fit a name pattern in a
% source folder, concatenates them, rotates and reflects them
% appropriately, creating a ROFEX (original) video.
% The image sequences correspond to a part of a full scan run (variable
% drum depth, constant angular position).
% All necessary parameters are read from the file parametersFul0.m, found
% in the output folder.
% The full image sequence is saved as the variable 'seq' in the file
% data.mat in the output folder. The experimental conditions, read from
% the script, are also saved in the same data.mat file.
% The full image sequence is also saved as the video 'original.avi' in the
% output folder.

% Theodoros-Nestor Papapetrou
% Last edited: 27 Jul 2022

%% Input

% outputFolder: the path to the output folder, where all results are saved.
    % The output folder needs to exist and contain the file
    % 'parametersFul0.m' with the correct parameter values before calling
    % this function.
    % e.g. outputFolder = 'P:\ROFEX data\202103\run1f_1rot';
    
%% Output

% seq: H x W x F uint8 array, the ROFEX video

%% Read parameters

run(fullfile(outputFolder,'parametersFul0.m'));

%% Load image sequence

sourceFiles = dir(fullfile(sourceFolder,pattern));

% Sort files in chronological order
for ii = 1:numel(sourceFiles)
    sourceFiles(ii).order = sscanf(sourceFiles(ii).name,sortingPattern);
    sourceFiles(ii).fullname = fullfile(sourceFiles(ii).folder,sourceFiles(ii).name);
    sourceFiles(ii).pages = numel(imfinfo(sourceFiles(ii).fullname));
end
sourceFiles = table2struct(sortrows(struct2table(sourceFiles), 'order'));

% Read files in chronological order
F = sum([sourceFiles.pages]); % total number of frames
[H,W] = size(imread(sourceFiles(1).fullname,1)); % height and width
seq = zeros([H,W,F],'uint8');
ii = 0;
for filenum = 1:numel(sourceFiles)
    for pagenum = 1:sourceFiles(filenum).pages
        ii = ii + 1;
        seq(:,:,ii) = imread(sourceFiles(filenum).fullname,pagenum);
    end
end

%% Rotation and reflection

seq = flip(rot90(seq,rotations),flipdimension);

%% Save results in data.mat

data = fullfile(outputFolder,'data.mat');
if isfile(data)
    save(data,'seq','experimentalConditions','-append');
else
    save(data,'seq','experimentalConditions');
end

%% Save full image sequence as video

originalvid = permute(seq,[1 2 4 3]);
outputFile = 'original.avi';
outputFilename = fullfile(outputFolder,outputFile);
writeVideo2Path(originalvid,outputFilename,25);
