function UFXCTProcessingFul1(outputFolder)

% Loads a full ROFEX image sequence (variable 'seq' in data.mat in
% the source folder), and applies a circle mask in all frames. Also
% calculates the relative depth for every frame.
% All necessary parameters are read from the file parametersFul1.m, found
% in the output folder.
% The file data.mat is copied from the source folder to the output
% folder. The new masked image sequence ('seqcircle'), the characteristic
% frames of the image sequence ('characteristicFrames'), the particle
% diameter as determined from a reference frame
% ('particleDiameter_pixels'), the parameters of the circle mask
% ('drum') and the calculated relative depth ('calculations') are appended
% to data.mat in the output folder.
% The new masked image sequence is also saved as a video ('masked.avi') in
% the output folder.

% The following code is used for the fitting of the circle:

% Nikolai Chernov (2020). Circle Fit (Pratt method)
% (https://www.mathworks.com/matlabcentral/fileexchange/22643-circle-fit-pratt-method),
% MATLAB Central File Exchange. Retrieved August 15, 2019.

% Theodoros-Nestor Papapetrou
% Last edited: 17 Oct 2022

%% Input

% outputFolder: the path to the output folder, where all results are saved.
    % The output folder needs to exist and contain the file
    % 'parametersFul1.m' with the correct parameter values before calling
    % this function.
    % e.g. outputFolder = 'P:\ROFEX data\202103\run1f_1rot';

%% Read parameters

run(fullfile(outputFolder,'parametersFul1.m'));

%% Copy data.mat to output folder, load data

if ~isequal(sourceFolder,outputFolder)
    data = fullfile(sourceFolder,'data.mat');
    copyfile(data,outputFolder)
end

data = fullfile(outputFolder,'data.mat');
load(data,'seq');
[H,W,F] = size(seq);

%% Segment the chosen frame and keep the chosen level

img = seq(:,:,frameChosen); % chosen frame with no noise outside drum
thresh = multithresh(img,noOfThresholds);
seg_img = imquantize(img,thresh);
mask = seg_img == level;

%% Detect the top and bottom surface in the selected level
% This should be the outside of the drum circle

Cols = zeros(W,1);
TopRows = zeros(W,1);
BottomRows = zeros(W,1);
k = 0;
for col = 1:W
    maskcol = mask(:,col);
    if sum(maskcol) > 0
        k = k + 1;
        Cols(k) = col;
        TopRows(k) = find(maskcol == 1,1);
        BottomRows(k) = find(maskcol == 1,1,'last');
    end
end
drumPixels = [Cols(1:k) TopRows(1:k); Cols(1:k) BottomRows(1:k)];

%% Fit drum circle

drum = CircleFitByPratt(drumPixels); % [col_center, row_center, radius]
col_center = drum(1);
row_center = drum(2);

% The radius of the fitted circle is decreased to include only the inside
% of the drum and exclude the drum wall
radius = drum(3) - radiusDecrease;
drum(3) = radius;

%% Show fitted circle

% theta = linspace(0,2*pi,1000);
% col_circle = col_center + radius*sin(theta);
% row_circle = row_center + radius*cos(theta);
% 
% figure
% imshow(img);
% hold on;
% plot(col_circle,row_circle,'LineWidth',2);

%% Mask sequence with fitted circle

maskCircle = circle([H W],[row_center col_center] - 0.5,radius,'logical');
seqcircle = zeros(size(seq),'uint8');
for ii = 1:F
    seqcircle(:,:,ii) = mask_rgb_image(seq(:,:,ii),maskCircle);
end

%% Calculate relative depth for each frame

front = characteristicFrames.frontWall;
back = characteristicFrames.backWall;
calculations.relativeDepth = ((1:F) - front)./(back - front);

%% Save results in data.mat

save(data,'seqcircle','characteristicFrames','particleDiameter_pixels',...
    'drum','calculations','-append');

%% Save masked image sequence as video

maskedvid = permute(seqcircle,[1 2 4 3]);
outputFilename = fullfile(outputFolder,'masked.avi');
writeVideo2Path(maskedvid,outputFilename,25);
