function UFXCTProcessingFul2(outputFolder)

% Loads a circle-masked ROFEX image sequence (variable 'seqcircle' in
% data.mat in the source folder), and segments it into black-red-white
% by applying two thresholds, equal for all frames.
% All necessary parameters are read from the file parametersFul2.m, found
% in the output folder.
% The file data.mat is copied from the source folder to the output
% folder. The segmented image sequence ('seqPvid') the parameters of the
% segmentation ('segmentationParameters') and the frame range where the
% segmentation is meaningful ('characteristicFrames') are appended to
% data.mat in the output folder.
% The segmented image sequence is also saved as a video ('Pvid.avi') in
% the output folder. A side-by-side video of the segmented and the masked
% image sequences ('comparison.avi') is also saved in the output folder. A
% shorter version of these videos ('PvidGranular.avi',
% 'comparisonGranular.avi'), limited to the meaningful frame range, is also
% saved in the output folder.

% Theodoros-Nestor Papapetrou
% Last edited: 10 Oct 2022

%% Input

% outputFolder: the path to the output folder, where all results are saved.
    % The output folder needs to exist and contain the file
    % 'parametersFul2.m' with the correct parameter values before calling
    % this function.
    % e.g. outputFolder = 'P:\ROFEX data\202103\run1_part1';

%% Read parameters

run(fullfile(outputFolder,'parametersFul2.m'));

%% Copy data.mat to output folder, load data

if ~isequal(sourceFolder,outputFolder)
    data = fullfile(sourceFolder,'data.mat');
    copyfile(data,outputFolder)
end

data = fullfile(outputFolder,'data.mat');
load(data,'seqcircle','characteristicFrames');
[~,~,F] = size(seqcircle);

%% Save meaningful frame range

meaningfulRange = sort(meaningfulRange);
characteristicFrames.meaningfulRange = meaningfulRange;

%% Create red and white binary masks and segmented image sequence

thresholds = segmentationParameters.thresholds;
seqPvid = zeros(size(seqcircle),'uint8');
for ii = 1:F
    img = seqcircle(:,:,ii);
    maskNonBlack = imbinarize(img,thresholds(1)*1./255.);
    maskWhite = imbinarize(img,thresholds(2)*1./255.);
    % maskRed = maskNonBlack > maskWhite;
    seqPvid(:,:,ii) = uint8(maskNonBlack) + uint8(maskWhite);
end

%% Save results in data.mat

save(data,'seqPvid','segmentationParameters','characteristicFrames',...
    '-append');

%% Save segmented image sequence as video

seqPvidRGB = Pvid2RGB(seqPvid);
outputFilenameP = fullfile(outputFolder,'Pvid.avi');
writeVideo2Path(seqPvidRGB,outputFilenameP,25);

%% Save comparison video

maskedvid = permute(seqcircle,[1 2 4 3]);
comparison = [cat(3,maskedvid,maskedvid,maskedvid) seqPvidRGB];
outputFilenameP = fullfile(outputFolder,'comparison.avi');
writeVideo2Path(comparison,outputFilenameP,25);

%% Save short segmented image sequence as video

frameRange = meaningfulRange(1):meaningfulRange(end);
seqPvidGran = seqPvid(:,:,frameRange);
seqPvidGranRGB = Pvid2RGB(seqPvidGran);
outputFilenameP = fullfile(outputFolder,'PvidGranular.avi');
writeVideo2Path(seqPvidGranRGB,outputFilenameP,25);

%% Save short comparison video

comparisonGran = comparison(:,:,:,frameRange);
outputFilenameP = fullfile(outputFolder,'comparisonGranular.avi');
writeVideo2Path(comparisonGran,outputFilenameP,25);
