function UFXCTProcessingFul3(outputFolder)

% Loads a segmented image sequence (variable 'seqPvid' in data.mat in
% the source folder) and calculates (1) the mixing index, using (a) the
% modified contact method and (b) the variance method, (2) the average red
% concentration, and (3) the filling degree (bulk area divided by drum
% area), as a function of relative drum depth, only for the meaningful
% frame range. The order of the calculated values follows the order of
% frames, NOT the order of relative depth.
% All necessary parameters are read from the file parametersFul3.m, found
% in the output folder.
% The file data.mat is copied from the source folder to the output
% folder. The calculated quantites ('calculations') are appended to
% data.mat in the output folder.
% A plot of the modified contact method mixing index ('M.png'), a plot of
% the average red concentration ('c.png') and a plot of the filling degree
% ('f.png'), all as a function of relative drum depth, are also saved in
% the output folder.

% Theodoros-Nestor Papapetrou
% Last edited: 27 Feb 2023

%% Input

% outputFolder: the path to the output folder, where all results are saved.
    % The output folder needs to exist and contain the file
    % 'parametersFul3.m' with the correct parameter values before calling
    % this function.
    % e.g. outputFolder = 'P:\ROFEX data\202103\run1_part1';

%% Read parameters

framesToOmit = [];
run(fullfile(outputFolder,'parametersFul3.m'));
    
%% Copy data.mat to output folder, load data

if ~isequal(sourceFolder,outputFolder)
    data = fullfile(sourceFolder,'data.mat');
    copyfile(data,outputFolder)
end

data = fullfile(outputFolder,'data.mat');
load(data,'seqPvid','characteristicFrames','particleDiameter_pixels',...
    'calculations','drum');

%% Limit frame range to the meaningful frame range

meaningfulRange = characteristicFrames.meaningfulRange;
frameRange = meaningfulRange(1):meaningfulRange(end);
seqPvidGran = seqPvid(:,:,frameRange);
noOfFrames = numel(frameRange);

%% initialization

d_p = particleDiameter_pixels;
framesOmitted = [];
Mc_mod = NaN(1,noOfFrames); % mixing index - modified contact method
noOfRatios = numel(L_over_d);
Md = NaN(noOfRatios,noOfFrames); % mixing index - variance method
c = NaN(1,noOfFrames); % average red concentration
f = NaN(1,noOfFrames); % filling degree

%% Calculations
        
for ff = 1:noOfFrames
    omitted = ismember(ff,framesToOmit);
    if ~omitted
        P = seqPvidGran(:,:,ff);
        try
            Mc_mod(ff) = M_cont_modified(P,d_p);
            for ii = noOfRatios:-1:1
                ratio = L_over_d(ii);
                Md(ii,ff) = M_variance3D(P,d_p,ratio);
            end
            red_area = sum(sum(P==1)); % pixels
            solid_area = sum(sum(P>0)); % pixels
            c(ff) = red_area/solid_area;
            [~,f(ff)] = angleOfRepose_ROFEX(P,drum,d_p);
        catch
            omitted = true;
        end
    end
    if omitted
        framesOmitted = [framesOmitted ff];
    end
end

mixingIndex.contactModified = Mc_mod;
for ii = noOfRatios:-1:1
    ratio = L_over_d(ii);
    variance(ii).L_over_d = ratio;
    variance(ii).M = Md(ii,:);
end
mixingIndex.variance = variance;
calculations.mixingIndex = mixingIndex;
calculations.averageRedConcentration = c;
calculations.fillingDegree = f;
calculations.framesOmitted = framesOmitted;

%% Plot calculated values, save plots in output folder

rdd = calculations.relativeDepth(frameRange);

figure;
plot(rdd,Mc_mod);
xlabel('z/L')
ylabel('M');
saveas(gcf,fullfile(outputFolder,'M.png'));
close

figure;
plot(rdd,c);
xlabel('z/L')
ylabel('c');
saveas(gcf,fullfile(outputFolder,'c.png'));
close

figure;
plot(rdd,f);
xlabel('z/L')
ylabel('f');
saveas(gcf,fullfile(outputFolder,'f.png'));
close

%% Save results in data.mat

save(data,'calculations','-append');
