function UFXCTProcessingFul5(outputFolder)

% Collects all full-scan experimental (UFXCT) runs that satisfy some given
% experimental conditions (filling degree, rotational speed, number of
% rotations) and saves the calculated data for the meaningful frame range
% in a common data.mat file in the output folder.
% All necessary parameters are read from the file parametersFul5.m, found
% in the output folder.

% Theodoros-Nestor Papapetrou
% Last edited: 27 Feb 2023

%% Input

% outputFolder: the path to the output folder, where all results are saved.
    % The output folder needs to exist and contain the file
    % 'parametersFul5.m' with the correct parameter values before calling
    % this function.
    % e.g. outputFolder = 'P:\ROFEX data\202103\static_full\f0p50';

%% Read parameters

run(fullfile(outputFolder,'parametersFul5.m'));

f = fillingDegree;
w = rotationalSpeed_rpm;
n = noOfRotations;

%% Find runs

% Read all files with runs
sourceFiles = dir(sourceFolder);

% Remove '.' and '..' from list
idx = ismember({sourceFiles.name},{'.','..'});
sourceFiles = sourceFiles(~idx);

% Remove non-folders from list
idx = [sourceFiles(:).isdir];
sourceFiles = sourceFiles(idx);
noOfFiles = numel(sourceFiles);

% Find all runs with given experimental conditions
idx = true(1,noOfFiles);
for ii = 1:noOfFiles
    try
        fileName = fullfile(sourceFolder,...
            sourceFiles(ii).name);
        load(fullfile(fileName,'data.mat'),'experimentalConditions');
        eC = experimentalConditions;
        if ~ismembertol(eC.fillingDegree,f)
            idx(ii) = false;
        elseif ~ismembertol(eC.rotationalSpeed_rpm,w)
            idx(ii) = false;
        elseif ~ismembertol(eC.rotationsDone,n)
            idx(ii) = false;
        end
    catch   % if any error occurs, e.g. there is no 'data.mat' file,
            % the corresponding file is discarded
        idx(ii) = false;
    end
end
sourceFiles = sourceFiles(idx);
noOfFiles = numel(sourceFiles);

%% Calculate constants

clear f w n
constants.drumDiameter_cm = drumDiameter_cm;
constants.drumLength_cm = drumLength_cm;
constants.particleDiameter_mm = particleDiameter_mm;
drumRadius_m = drumDiameter_cm / 200;
g_SI = 9.81;

%% Collect data

if noOfFiles > 0
    for ii = noOfFiles:-1:1
        
        % Load data
        fileName = fullfile(sourceFolder,sourceFiles(ii).name);
        data = load(fullfile(fileName,'data.mat'));
        eC = data.experimentalConditions;
        cF = data.characteristicFrames;
        meaningfulRange = cF.meaningfulRange;
        frameRange = meaningfulRange(1):meaningfulRange(end);
        calc = data.calculations;
        
        % Save experimental conditions
        runData(ii).fillingDegree = eC.fillingDegree;
        
        w = eC.rotationalSpeed_rpm;
        runData(ii).rotationalSpeed_rpm = w;
        rotationalSpeed_SI = w*pi/30;
        Froude = (rotationalSpeed_SI)^2*drumRadius_m/g_SI;
        runData(ii).Froude = Froude;
        
        runData(ii).rotations = eC.rotationsDone;
        
        % Save calculations
        rdd = calc.relativeDepth(frameRange);
        z = rdd * drumLength_cm;
        runData(ii).depth_cm = z;
        runData(ii).relativeDepth = rdd;
        
        runData(ii).mixingIndex = calc.mixingIndex;
        
        c = calc.averageRedConcentration;
        runData(ii).averageRedConcentration = c;
        
        fc = calc.fillingDegree;
        runData(ii).fillingDegreeCalc = fc;
    end
end

%% Save results to data.mat

data = fullfile(outputFolder,'data.mat');
if isfile(data)
    save(data,'constants','runData','-append');
else
    save(data,'constants','runData');
end
