function outputData = angVelocityPosition(inputData)

% Calculates the angular velocity, angular position and time for every
% frame, as well as the frame when the drum stops rotating.

% Theodoros-Nestor Papapetrou
% Last edited: 29 Jul 2022

%% Input

% inputData: structure containing the following fields:
    % noOfFrames: the total number of frames
    % wNominal: the nominal value of angular velocity, in rpm
    % noOfRotations: the total number of rotations
    % DtAcc: the acceleration time, in s
    % DtDec: the deceleration time, in s
    % frameRate: the frame rate, in Hz (frames/s)
    % startingFrame: the last frame before the drum starts rotating, >= 1
    
    % Example values:
    % inputData.noOfFrames = 1500;
    % inputData.wNominal = 45;
    % inputData.noOfRotations = 10;
    % inputData.DtAcc = 0.1;
    % inputData.DtDec = 0.1;
    % inputData.frameRate = 100;
    % inputData.startingFrame = 90;
    
%% Output

% outputData: structure containing the following fields (F: noOfFrames):
    % w: 1 x F vector, angular velocity of each frame, in Hz
    % wRPM: 1 x F vector, angular velocity of each frame, in rpm
    % theta: 1 x F vector, angular position of each frame, in rotations
    % tAbs: 1 x F vector, time of each frame where t=0 is the first frame,
        % in s
    % tRot: 1 x F vector, time of each frame where t=0 is the last frame
        % before the drum starts rotating, in s
    % stoppingFrameNumber: integer, the frame when the drum stops rotating

%% Initialization

F = inputData.noOfFrames;
w_Hz = inputData.wNominal / 60;
U = inputData.noOfRotations;
t_acc = inputData.DtAcc;
t_dec = inputData.DtDec;
frate = inputData.frameRate;
totalRotationTime = U / w_Hz + (t_acc + t_dec) / 2;
w = zeros(1,F);
theta = zeros(1,F);
dt = 1 / frate; % time interval between consecutive frames, in s

%% Time vectors

frameAbs = (1:F) - 1;
tAbs = frameAbs * dt;
frameRot = (1:F) - inputData.startingFrame;
tRot = frameRot * dt;

%% Angular velocity and position vectors

for ii = 1:F
    if tRot(ii) <= 0 % before the rotation has begun
        w(ii) = 0;
        theta(ii) = 0;
    elseif tRot(ii) <= t_acc % acceleration
        w(ii) = w_Hz / t_acc * tRot(ii);
        theta(ii) = theta(ii-1) + (w(ii) + w(ii-1))*dt/2;
    elseif tRot(ii) <= totalRotationTime - t_dec % constant ang. velocity
        w(ii) = w_Hz;
        if tRot(ii) - t_acc >= dt
            theta(ii) = theta(ii-1) + w_Hz*dt;
        else
            dt2 = tRot(ii) - t_acc;
            dt1 = dt - dt2;
            theta(ii) = theta(ii-1) + (w(ii) + w(ii-1))*dt1/2 + w_Hz*dt2;
        end
    elseif tRot(ii) <= totalRotationTime % deceleration
        w(ii) = w_Hz / t_dec * (totalRotationTime - tRot(ii));
        if tRot(ii) - (totalRotationTime - t_dec) >= dt
            theta(ii) = theta(ii-1) + (w(ii) + w(ii-1))*dt/2;
        else
            dt2 = tRot(ii) - (totalRotationTime - t_dec);
            dt1 = dt - dt2;
            theta(ii) = theta(ii-1) + w_Hz*dt1 + (w(ii) + w(ii-1))*dt2/2;
        end
    else % after the rotation has stopped
        w(ii) = 0;
        if tRot(ii) - totalRotationTime >= dt
            theta(ii) = theta(ii-1);
        else
            dt2 = tRot(ii) - totalRotationTime;
            dt1 = dt - dt2;
            theta(ii) = theta(ii-1) + (w(ii) + w(ii-1))*dt1/2;
        end
    end
end

wRPM = w * 60;

%% Stopping frame

stoppingFrameNumber = find(theta == max(theta),1);

%% Output structure

outputData.w = w;
outputData.wRPM = wRPM;
outputData.theta = theta;
outputData.tAbs = tAbs;
outputData.tRot = tRot;
outputData.stoppingFrameNumber = stoppingFrameNumber;
