function [a_repose, f, Cols, Rows, polyn, R2] = ...
    angleOfRepose_ROFEX(P,drum,d_p)

% Calculates the average dynamic angle of repose, in degrees, and the
% filling degree, for an image of the rotating drum, by approximating the
% free surface with a 1st order polynomial. The image should not have any
% noise above the free surface.

% Theodoros-Nestor Papapetrou
% Last edited: 17 Feb 2023

%% Input

% P: uint8 H x W matrix with values corresponding to colors:
% 1: red, 2: white, 0: background (black)
% P represents an image of the rotating drum with red and white particles

% Remark: for the calculations of this function, the actual values of the
% elements of P do not matter, as long as the background has the value 0
% and the foreground (granular particles) have nonzero values.

% drum: 1x3 double array, contains the parameters of the rotating drum:
    % drum(1): column of the center of the drum (in pixels)
    % drum(2): row of the center of the drum (in pixels)
    % drum(3): radius of the drum (in pixels)
    
% d_p: positive scalar, particle diameter (in pixels)

%% Output

% a_repose: the average angle of repose (scalar, in degrees). It can have a
% negative value if the drum rotates counterclockwise

% f: volumetric filling degree, i.e. the ratio of bulk area over total drum
% area (scalar between 0 and 1)

% Cols: the columns of the pixels belonging to the free surface

% Rows: the rows of the pixels belonging to the free surface

% polyn = [s b]: the coefficients of the 1st-order polynomial approximation
% of the free surface (s: slope, b: abscissa; scalars)

% R2: the R^2 coefficient of the approximation (scalar between 0 and 1)

%% Extract pixels of free surface
[Cols,Rows] = freeSurfacePixels(P,drum,d_p);

%% Linear regression - Cols as x, Rows as y

polyn = polyfit(Cols,Rows,1);
s = polyn(1); % slope
b = polyn(2); % abscissa
a_repose = atand(-s); % negative sign because the positive row-axis points
                      % downward

%% Filling degree

c0 = drum(1); % column coordinate of center
r0 = drum(2); % row coordinate of center
r_drum = drum(3); % radius of drum circle

% signed distance of drum center to free surface
dist = (s*c0 - r0 + b)/sqrt(s^2 + 1);
% dist > 0 if the center lies above the free surface
% dist < 0 if the center lies below the free surface

h = r_drum - dist; % average height of bed, if tilted horizontally
phi = 2*acos(1 - h/r_drum); % central angle of the bed
f = (phi - sin(phi))/(2*pi);

%% R^2 of linear regression

%https://de.mathworks.com/help/matlab/data_analysis/linear-regression.html
yfit = polyval(polyn,Cols);
yresid = Rows - yfit;
SSresid = sum(yresid.^2);
SStotal = (length(Rows)-1) * var(Rows);
R2 = 1 - SSresid/SStotal;