function [Cols,Rows] = freeSurfacePixels(P,drum,d_p)

% Extracts the pixels of the free surface from an image of the rotating
% drum.
% A pixel in the top surface of P is considered to belong to the free
% surface if its distance from the circumference of the drum circle is more
% that one particle diameter.

% Theodoros-Nestor Papapetrou
% Last edited: 23 May 2022

%% Input

% P: HxW matrix, image of the rotating drum. Particles are represented by
% non-zero pixel values and the background is represented by zero pixel
% values.

% drum: 3-element vector, containing the parameters of the drum circle
% associated with P:
    % drum(1): column of the center of the drum (in pixels)
    % drum(2): row of the center of the drum (in pixels)
    % drum(3): radius of the drum (in pixels)
    
% d_p: diameter of the particles (in pixels)

%% Output

% Cols: 1-column vector, columns of the free surface pixels

% Rows: 1-column vector, rows of the free surface pixels

%% Initialization
P = logical(P);
W = size(P,2);
colCen = drum(1);
rowCen = drum(2);
drumR = drum(3);
Cols = zeros(W,1);
Rows = zeros(W,1);
k = 0;
distMax = drumR - d_p;

%% Extract pixels of free surface
for jj = 1:W
    col = P(:,jj);
    if sum(col) > 0 % non-empty column
        ii = find(col,1); % row of first particle pixel
        distance = norm([jj - colCen, ii - rowCen]);
        if distance < distMax
            k = k + 1;
            Cols(k) = jj;
            Rows(k) = ii;
        end
    end
end
Cols = Cols(1:k);
Rows = Rows(1:k);
