function parameters = modelParameters(xdata,ydata,eqn)

% Returns the model parameters that best fit the data in a least-square
% sense.

% Theodoros-Nestor Papapetrou
% Last edited: 04 Nov 2022

%% Input

% xdata, ydata: column vectors of same size

% eqn: integer, indicating the model used:
    % 0: first order linear differential equation, zero initial value:
        % y = y_inf - y_inf*exp(-x/T)
    % 1: first order linear differential equation:
        % y = y_inf + (y0 - y_inf)*exp(-x/T)
    % 2: second order linear differential equation, zero initial values:
        % T^2(d^2y/dx^2) + 2zT(dy/dx) + y = K, y(0) = 0, dy/dt(t=0) = 0
    % 3: second order linear differential equation:
        % T^2(d^2y/dx^2) + 2zT(dy/dx) + y = K, y(0) = y0, dy/dt(t=0) = dy0
        
%% Output

% parameters: the model parameters
    % If eqn = 0: [T y_inf]
    % If eqn = 1: [T y_inf y_0]
    % If eqn = 2: [T z K]
    % If eqn = 3: [T z K y0 dy0]

%%

if eqn == 0
    fun = @(p,xdata) p(2) + (0-p(2)).*exp(-xdata/p(1));
    p0 = [1 xdata(end)];
    lb = [0 -Inf];
    ub = [Inf Inf];
elseif eqn == 1
    fun = @(p,xdata) p(2) + (p(3)-p(2)).*exp(-xdata/p(1));
    p0 = [1 xdata(end) 0];
    lb = [0 -Inf -abs(xdata(end)-xdata(1))];
    ub = [Inf Inf abs(xdata(end)-xdata(1))];
elseif eqn == 2
    fun = @(p,xdata) solve2ndorderLinDE(p(1),p(2),p(3),pulse(xdata),xdata,[0;0]);
    p0 = [1 1 xdata(end)];
    lb = [0 0 -Inf];
    ub = [Inf Inf Inf];
else
    fun = @(p,xdata) solve2ndorderLinDE(p(1),p(2),p(3),pulse(xdata),xdata,[p(4);p(5)]);
    p0 = [1 1 xdata(end) 0 0];
    lb = [0 0 -Inf -abs(xdata(end)-xdata(1)) -Inf];
    ub = [Inf Inf Inf abs(xdata(end)-xdata(1)) Inf];
end

options = optimoptions('lsqcurvefit','Display','off');
parameters = lsqcurvefit(fun,p0,xdata,ydata,lb,ub,options);

end

function pls = pulse(x)

pls = [x x];
pls(1,2) = 0;
pls(2:end,2) = 1;
    
end