function [A_right,len,c_right,A_perc] = rightArea(R,h,shape,params)

% Calculates the area of the right region of a rotating drum of radius R
% and uniform filling height h, in the artificial limiting distribution
% where the two granular species are perfectly segregated in a left and a
% right region. The right region is considered to have the rarest of the
% two species, with a fraction (c) at most half of the total area under the
% free surface.
% Two cases are considered: (a) the two regions are separated by a straight
% line, (b) the two regions are separated by a circular arc.

% Theodoros-Nestor Papapetrou
% Last edited: 05 Oct 2020

%% Input

% R: the radius of the rotating drum (positive double scalar)
% h: the filling height of the drum; positive double scalar, 0<h<2R
% shape: denotes the shape of the separating curve; character with possible
    % values: "line", "arc".
% params: geometric parameters of the separating curve, different for each
    % shape (double array)
    % If shape is "line":
        % params(1) = x: the x-coordinate of the intersection of the
            % horizontal free surface and the separating line (point B).
            % The origin of the coordinate system is the center of the drum
            % [double scalar, abs(x) must be smaller than sqrt(h*(2*R-h))]
        % params(2) = eta: the angle between the separating line and
            % negative y-axis (double scalar, 0<=eta<pi/2 in radians)
    % If shape is "arc":
        % params(1) = xP: the x-coordinate of the center of the separating
            % arc (double scalar)
        % params(2) = yP: the y-coordinate of the center of the separating
            % arc (double scalar)
        % params(3) = r: the radius of the separating arc (double scalar)
        % The following constraints must all be true:
            % xP > 0 (the center of the arc is to the right of the center
                % of the drum
            % xP^2 + yP^2 > (R-r)^2 (the two circles are not one inside
                % the other)
            % xP^2 + yP^2 < (R+r)^2 (the two circles are not one outside
                % the other)
            % abs(h-R-yP) < r (the arc intersects the free surface at two
                % points)
            % (sqrt(R^2-(h-R)^2)-xP)^2 + (h-R-yP)^2 <= r^2 (the right
                % intersection of the free surface with the drum circle is
                % inside the arc)
    
%% Output

% A_right: the area of the right region (positive double scalar)
% len: the length of the separating curve (positive double scalar)
% c_right: the overall granular concentration of the species in the right
    % area; equivalently, the ratio of the right area to the total area of
    % the granular bed, i.e. the area under the free surface; double
    % scalar, 0<c_right<1
% A_perc: the area of the right region as a percentage of the total area of
    % the drum circle; double scalar, 0<A_perc<1

%% Check input ranges

if R <= 0
    error('The drum radius R must be positive.')
end

if abs(h-R) > R
    error('The filling height h must be positive and less than the drum diameter.')
end

y0 = h - R; % y-coordinate of the free surface
xA = sqrt(R^2 - y0^2); % x-coordinate of the right intersection of the free
                       % surface with the drum circle (point A)
% disp(['xA: ' num2str(xA)])

if strncmpi(shape,'line',1)
    shape = 'line';
elseif strncmpi(shape,'arc',1)
    shape = 'arc';
else
    error('Non-existent method.')
end

imagflag = false; % logical value indicating if any coordinate is a
                      % complex number

if isequal(shape,'line')
    x = params(1);
    eta = params(2);
    
    if y0 < 0
    else % y0 >= 0
    end
    
    if abs(x) > xA
        error('The intersection of the separating line and the free surface falls outside the drum.')
    end
    
    if any([eta<0, eta>pi()/2])
        error('The angle (in radians) of the separating line must be nonnegative and smaller than pi/2.')
    end
    
else % shape is 'arc'
    xP = params(1);
    yP = params(2);
    r = params(3);
    
    if xP <= 0
        error('The center of the arc is not to the right of the drum center.')
    end
    
    if xP^2 + yP^2 <= (R-r)^2
        % warning('The two circles are one inside the other.')
        imagflag = true;
    end
    
    if xP^2 + yP^2 >= (R+r)^2
        % warning('The two circles are one outside the other.')
        imagflag = true;
    end
    
    if abs(y0 - yP) >= r
        % warning('The arc does not intersect the free surface at two points.')
        imagflag = true;
    end
    
    if (xA - xP)^2 + (y0 - yP)^2 > r^2
        % warning('The right intersection of the free surface with the drum circle is outside the arc.')
        imagflag = true;
    end
    
end

%% Calculate the coordinates of C and len
% C is the intersection point between the separating curve and the drum
% circle, below the free surface.

if isequal(shape,'line')
    
    if eta == 0
        
        xC = x; % x-coordinate of C
        yC = -sqrt(R^2 - x^2); % y-coordinate of C
        len = y0 - yC;
        
    else % eta > 0
        
        s = -cot(eta); % slope of the separating line
        b = y0 - s*x; % abscissa of the separating line
        xC = (sqrt(R^2*(s^2+1)-b^2)-s*b)/(s^2+1);
        yC = s*xC + b;
        len = (xC - x)*sqrt(s^2+1);
        
    end
    
else % shape == 'arc'
    
    if imagflag == true
        
        len = 4*R; % too big value
        
    else % no complex coordinates
    
        x = xP - sqrt(r^2 - (y0-yP)^2); % the x-coordinate of the intersection
                                        % of the horizontal free surface and
                                        % the separating line (point B).

        [xC,yC] = circleIntersections(0,0,xP,yP,R,r);
        % disp(['xC: ',num2str(xC)])
        % disp(['yC: ',num2str(yC)])

        if y0 >= yP
            ksiB = atan2(y0 - yP, x - xP); % angle of B measured from the arc
                                           % center
        else % y0 < yP
            ksiB = 2*pi() + atan2(y0 - yP, x - xP);
        end
        % pi/2 < ksiB < 3pi/2

        ksiC = 2*pi() + atan2(yC - yP, xC - xP); % angle of C measured from the
                                                 % arc center
        % pi < ksiC < 2pi, ksiC > ksiB

        ksi = ksiC - ksiB; % angle of the circular arc
        % disp(['ksi: ' num2str(ksi/pi()*180) ' degrees'])
        len = r*ksi;
        
    end
    
end

%% Calculate the area

if imagflag == true
    
    A_right = 0;
    
else

    thetaA = atan2(y0,xA); % angle of point A measured from the drum circle
    % -pi/2 < thetaA < pi/2
    % disp(['thetaA: ' num2str(thetaA/pi()*180) ' degrees'])


    thetaC = atan2(yC,xC); % angle of point C measured from the drum circle
    % -pi < thetaC < pi/2, thetaC < thetaA
    % disp(['thetaC: ' num2str(thetaC/pi()*180) ' degrees'])

    theta = thetaA - thetaC; % angle of the drum arc belonging to the right
                             % region

    A_right = R^2*theta + y0*(xA - x);

    if isequal(shape,'line')

        if eta == 0

            A_right = A_right - x*(y0 - yC);

        else % eta > 0

            A_right = A_right - b*(xC - x);

        end

    else % shape == 'arc'

        A_right = A_right - xP*(y0 - yC) - yP*(xC - x) + r^2*ksi;

    end

    A_right = A_right/2;
    
end

%% Calculate c_right and A_perc

phi = 2*acos(-y0/R); % angle of the total filled area arc
A_bed = R^2*(phi - sin(phi))/2; % area of the granular bed
c_right = A_right/A_bed;
A_perc = A_right/(pi()*R^2);