function sensitivityAnalysis(outputFolder)

% Performs a sensitivity analysis for the dependence of the average
% concentration, mixing index, dynamic angle of repose and calculated
% filling degree on the threshold values and the particle diameter.
% All necessary parameters are read from the file
% parametersSensitivity.m, found in the output folder.
% The results are saved in the file 'results.mat' in the output folder.
% Graphs of each dependent quantity as a function of each of the
% independent variables are saved in the output folder.

% Theodoros-Nestor Papapetrou
% Last edited: 20 May 2024

%% Input

% outputFolder: the path to the output folder.
    % The output folder needs to exist and contain the file
    % 'parametersSensitivity.m' with the correct parameter values before
    % calling this function.
    % e.g. outputFolder = '\\gssnas\bigdata\partmix\paper2\PT_special\data\dynamic\UFXCT\sensitivity';
    
%% Read parameters

imageFolder = {};
vidFrame = [];
xVariable = {};
xLabel = {};
yVariable = {};
yLabel = {};
xRadius = [];
particleDiameterFrequency = 0.1;
xError = [];
run(fullfile(outputFolder,'parametersSensitivity.m'));
noOfImages = numel(vidFrame);
noOfIndependentParameters = numel(xVariable);
noOfDependentQuantities = numel(yVariable);

%% Initialization

xFrequency = [1 1 particleDiameterFrequency];

for ii = noOfIndependentParameters:-1:1
    shiftedX = 0:xFrequency(ii):xRadius(ii);
    shiftedXValue{ii} = unique([shiftedX -shiftedX]);
end

%% Perform sensitivity analysis on each image

for kk = noOfImages:-1:1
    
    %% Load data
    
    dataFile = fullfile(imageFolder{kk},'data.mat');
    load(dataFile,'seqcircle','drum','segmentationParameters','particleDiameter_pixels');
    grayImage = seqcircle(:,:,vidFrame(kk));
    thresholds = segmentationParameters.thresholds;
    xOriginal = [thresholds particleDiameter_pixels];
    
    %% Calculations for each independent parameter (x)
    
    for ii = noOfIndependentParameters:-1:1

        % Possible values of perturbed independent variable
        xValue{ii} = xOriginal(ii) + shiftedXValue{ii};
        perturbationData(ii).xVariable = xVariable{ii};
        perturbationData(ii).xValue(kk,:) = xValue{ii};
        
        %% Petrurb x
        
        for pp = numel(xValue{ii}):-1:1
            
            % Find current parameter values
            xCurrent = [xOriginal drum];
            xCurrent(ii) = xValue{ii}(pp);
            
            % Calculate results
            y = calculateResults(grayImage,xCurrent);
            
            % Save results
            for jj = noOfDependentQuantities:-1:1
                perturbationData(ii).output(jj).yVariable = yVariable{jj};
                perturbationData(ii).output(jj).yValue(kk,pp) = y(jj);
            end
            
        end
        
        %% Calculate the slope of each dependent quantity with respect to x 
        
        for jj = noOfDependentQuantities:-1:1
            polyn = polyfit(xValue{ii},perturbationData(ii).output(jj).yValue(kk,:),1);
            perturbationData(ii).output(jj).slope(kk,1) = polyn(1);
        end
        
    end
    
end

%% Aggregate results

for jj = noOfDependentQuantities:-1:1
    totalSqError = 0;
    for ii = noOfIndependentParameters:-1:1
        meanSlope(ii,jj) = mean(perturbationData(ii).output(jj).slope(:));
        totalSqError = totalSqError + (meanSlope(ii,jj)*xError(ii))^2;
    end
    ii = noOfIndependentParameters + 1;
    meanSlope(ii,jj) = sqrt(totalSqError);
end
meanSlope = array2table(meanSlope,'RowNames',{'T1','T2','d_p','total_error'},...
    'VariableNames',{'f','a','c','M'});

%% Save results

filename = fullfile(outputFolder,'results.mat');
if isfile(filename)
    save(filename,'perturbationData','meanSlope','-append');
else
    save(filename,'perturbationData','meanSlope');
end

%% Plot output

fig = figure;

for ii = 1:noOfIndependentParameters
    for jj = 1:noOfDependentQuantities
        
        subplot(noOfIndependentParameters,noOfDependentQuantities,(ii-1)*noOfDependentQuantities + jj)
        hold on
        for kk = 1:noOfImages
            scatter(shiftedXValue{ii},perturbationData(ii).output(jj).yValue(kk,:));
            xlabel(xLabel{ii},'Interpreter','latex');
            ylabel(yLabel{jj},'Interpreter','latex');
        end
    
    end
end

fig.WindowState = 'maximized';
saveas(gcf,fullfile(outputFolder,'plots.png'));
close

end

function y = calculateResults(img,x)

thresholds = x(1:2);
d_p = x(3);
drum = x(4:6);

%% Segment image
maskNonBlack = imbinarize(img,thresholds(1)*1./255.);
maskWhite = imbinarize(img,thresholds(2)*1./255.);
P = uint8(maskNonBlack) + uint8(maskWhite); % ternary image

%% Calculate filling degree and dynamic angle of repose
[a, f] = angleOfRepose_ROFEX(P,drum,d_p);

%% Average PP concentration
red_area = sum(sum(P==1));
solid_area = sum(sum(P>0));
c = red_area/solid_area;

%% Mixing index
M = M_cont_modified(P,d_p);

%% Concatenate results
y = [f a c M];

end
