function [y, t, yprime] = solve2ndorderLinDE(T, z, K, t_w, tspan, y0)

% Numerically solves the differential equation:

% T^2(d^2y/dt^2) + 2zT(dy/dt) + y = K*w(t)

% using ode45.

%% Input

% T, z, K: scalar parameters defined in the equation. z should be
% nonnegative.

% t_w: it can be either a scalar value, similar to T/z/K, or a Nx2 matrix,
% the first column containing values of t and the second column the
% corresponding values of the time-dependent input w(t). w(t) is considered
% to change linearly between any two successive t points. All t points must
% be unique.

% tspan: double array, t points where y is evaluated. Direct input to
% ode45. All points in tspan must be included in the t range of t_w.

% y0: 2x1 matrix, initial conditions. Default value [0; 0]. Direct input to
% ode45.

%% Output

% y: the solution y of the differential equation. Direct output of ode45.

% t: the t points in the domain where y is evaluated (evaluation points).
% Direct output of ode45.

% yprime: the first derivative of y evaluated at the same points t. Direct
% output of ode45.

%% Check and format input

tspan = unique(tspan);

if numel(t_w) > 1
    
    if (size(t_w,2) ~= 2)
        error('t_w does not have two columns.');
    else
        
        tw1 = t_w(:,1);
        tw1unique = unique(tw1);
        if numel(tw1unique) < numel(tw1)
            error('Some t values in t_w are not unique.');
        else
            t_w = sortrows(t_w);
        end

        if min(tspan) < min(tw1unique)
            error('The minimum of tspan is less than the minimum t of t_w.');
        elseif max(tspan) > max(tw1unique)
            error('The maximum of tspan is greater than the maximum t of t_w.');
        end
        
    end
    
end

T = T(1);
z = z(1);
K = K(1);

if nargin < 6
    y0 = [0; 0];
else
    y0 = [y0(1); y0(2)];
end

[t,y] = ode45(@(t,y) myode(t, y, T, z, K, t_w),tspan,y0);
yprime = y(:,2);
y = y(:,1);

end

function dydt = myode(t, y, T, z, K, t_w)

%% Calculate w

if numel(t_w) == 1
    w = t_w;
else
    w = interp1(t_w(:,1),t_w(:,2),t);
end

%% Calculate dydt

dydt = [y(2); K*w - y(1)/T^2 - 2*z*y(2)/T];

end