% incomplete!!

function [Pvid,params,Pvid_prime] = BWRedWhite2P(BWRed,BWWhite,params,track)

% Applies a sequence of erosions followed by a sequence of dilations on
% each frame of the binary masks BWRed and BWWhite to remove noise, then
% resolves the conflict whenever a pixel belongs to both masks and combines
% the masks into the matrix P.

% Theodoros-Nestor Papapetrou
% Last edited: 08 Oct 2020

%% Input

% BWRed: H x W x F logical array, with 1 values where the original color
% video has red particles, where F is the number of frames. This is matrix
% K in the paper.

% BWWhite: H x W x F logical array, with 1 values where the original color
% video has white particles.  This is matrix Q in the paper.

% params: structure with the following fields:
%   noOfMorphOps: nonnegative integer, the number of erosions and dilations
%   strelWidth: positive integer, width of the square structuring element
%       used in the erosions and dilations. It should be an odd number at
%       least 3; otherwise, it is rounded down to the closest odd.
% If some of these fields does not exist or is empty or if the value is out
% of range, default values are used and these are returned and saved in
% params.

% track: nonnegative integer, indicates how often the current frame under
% process is displayed. If track equals 0 or is not given, nothing is
% displayed.

%% Output

% Pvid: H x W x F uint8 array with values corresponding to colors:
% 1: red, 2: white, 0: black (background)
% Each page of Pvid, Pvid(:,:,ii) represents an image of the rotating drum
% at frame ii with red and white particles

% params: same as the input, where the field values have been added or
% corrected if necessary.

% Pvid_prime: H x W x F uint8 array with values corresponding to colors:
% 1: red, 2: white, 0: black (background), 3: spurious (red or white)
% Each page of Pvid, Pvid(:,:,ii) represents an image of the rotating drum
% at frame ii with red and white particles

%% Check params.noOfMorphOps (default: 3)

if checkFieldname(params,'noOfMorphOps')
    noOfMorphOps = floor(params.noOfMorphOps);
else
    noOfMorphOps = 3; % default value
end

params.noOfMorphOps = noOfMorphOps;

%% Check params.strelWidth (default: 3)

if checkFieldname(params,'strelWidth')
    width = floor(params.strelWidth);
else
    width = 3; % default value
end

if width <= 3 % at least 3
    width = 3;
else % rounded down to the nearest odd integer
    width = width + mod(width,2) - 1;
end

params.strelWidth = width;

%% Check track

if nargin < 4
    track = 0; % default value
else
    track = max(0,floor(track));
end

%% Initialization

sz = size(BWRed);

% Check if BWRed, BWWhite come from a video (F > 1) or a single image
% (F = 1)
if numel(sz) == 2 % single image
    F = 1;
else % video
    F = sz(3);
end

Pvid = zeros(sz,'uint8');
Pvid_prime = zeros(sz,'uint8');

if track == 0 % no progress tracking
    
    if noOfMorphOps == 0 % no morphological operations
        
        for fr = 1:F
            Red = BWRed(:,:,fr);
            White = BWWhite(:,:,fr);
            P = mergeRedWhite(Red,White);
            Pvid_prime(:,:,fr) = P;
            Pvid(:,:,fr) = replaceMarked(P,3,[1 2],3);
        end
        
    else % with morphological operations
        
        for fr = 1:F
            Red = BWRed(:,:,fr);
            White = BWWhite(:,:,fr);
            SE = strel('square',width);
            Red = morphOps(Red,noOfMorphOps,SE);
            White = morphOps(White,noOfMorphOps,SE);
            P = mergeRedWhite(Red,White);
            Pvid_prime(:,:,fr) = P;
            Pvid(:,:,fr) = replaceMarked(P,3,[1 2],3);
        end
        
    end
    
else % with progress tracking
    
    strF = num2str(F);
    
    if noOfMorphOps == 0 % no morphological operations
        
        for fr = 1:F
            if mod(fr,track) == 0
                strfr = num2str(fr);
                disp(['Current frame: ',strfr,'/',strF])
            end
            Red = BWRed(:,:,fr);
            White = BWWhite(:,:,fr);
            P = mergeRedWhite(Red,White);
            Pvid_prime(:,:,fr) = P;
            Pvid(:,:,fr) = replaceMarked(P,3,[1 2],3);
        end
        
    else % with morphological operations
        
        for fr = 1:F
            if mod(fr,track) == 0
                strfr = num2str(fr);
                disp(['Current frame: ',strfr,'/',strF])
            end
            Red = BWRed(:,:,fr);
            White = BWWhite(:,:,fr);
            SE = strel('square',width);
            Red = morphOps(Red,noOfMorphOps,SE);
            White = morphOps(White,noOfMorphOps,SE);
            P = mergeRedWhite(Red,White);
            Pvid_prime(:,:,fr) = P;
            Pvid(:,:,fr) = replaceMarked(P,3,[1 2],3);
        end
        
    end
    
end

end

function BW = morphOps(BW,noOfMorphOps,SE)

% Performs a number of erosions followed by an equal number of dilations on
% the logical 2D matrix BW, using the given structuring element, SE.

for tt = 1:noOfMorphOps
    BW = imerode(BW,SE);
end

for tt = 1:noOfMorphOps
    BW = imdilate(BW,SE);
end

end

function P = mergeRedWhite(Red,White)

% Merges the two 2D binary masks Red and White into the 2D uint8 matrix P.
% P has value 0 where both masks are 0, 1 where only Red is 1, 2 where only
% White is 1 and 3 where both masks are 1.

P = uint8(Red) + 2*uint8(White);

end

function TF = checkFieldname(S,fieldname)

% Checks whether a given field in the structure S exists, whether it is
% nonempty, numeric, and whether the first element is nonnegative. If all
% these are true, TF = 1, otherwise TF = 0.

if ~isfield(S,fieldname)
    TF = 0;
else
    fld = S.(fieldname);
    if isempty(fld)
        TF = 0;
    elseif ~isnumeric(fld)
        TF = 0;
    else
        fld = fld(1);
        if fld < 0
            TF = 0;
        else
            TF = 1;
        end
    end 
end

end