function [M,Ms,Ma] = M_cont_modified(P,d_p,alternatives)

% Calculates the mixing index of an image using the modified contact
% method, by ignoring all background pixels separately in the row and
% column direction, followed by a normalization using a theoretical maximum
% and minimum value of contacts, dependent on the image and the diameter of
% the particles.

% Theodoros-Nestor Papapetrou
% Last edited: 08 Oct 2020

%% Input

% P: uint8 H x W matrix with values corresponding to colors:
% 1: red, 2: white, 0: background (black)
% P represents an image of the rotating drum with red and white particles

% d_p: the diameter of the balls in the rotating drum (in pixels),
% the same for both kinds of balls (scalar)

% alternatives: logical value, indicating whether the two alternatives
% considered in the supplementary materials should be calculated (1) or not
% (0). Default value: 0

%% Output

% M: the value of the mixing index, if the curve separating the two
% regions in the calculation of the minimum number of contacts (C_min) is a
% vertical straight line (double scalar). This is the version presented in
% the paper.

% Ms: the value of the mixing index, if the curve separating the two
% regions in the calculation of C_min is a straight line of optimal
% inclination, so as to minimize its length (double scalar). This is an
% alternative considered in the supplementary materials.

% Ma: the value of the mixing index, if the curve separating the two
% regions in the calculation of C_min is a circular arc, with its center at
% the right intersection point of the drum circle and the free surface
% (double scalar). This is an alternative considered in the supplementary
% materials.

% All three values are between 0 and 1.

%% Preliminary calculations

if nargin < 3
    alternatives = 0;
end

P = remove_0_cols_rows(P);
[H,W] = size(P);
c_avg = find_c_avg(P);

[~, ~, drum_circle, h, ~, bulk_area] = fillingDegree1(P);

r = drum_circle(3); % drum radius (in pixels)
% h: average height of the bed (in pixels)
% bulk_area: bulk area of the bed (in pixels)

%% Calculate maximum possible number of contacts, C_max

color_area = sum(sum(P~=0)); % number of non-black pixels

% Any of the following two alternatives for the calculation of circle_area
% can be used. The difference between the two values gets smaller as the
% pixel-per-particle ratio increases.

% circle_area = pi()*ball_diameter^2/4;
circle_area = sum(sum(circle(d_p))); % area of a single particle

C_max = 4*d_p*color_area/circle_area*min(c_avg,1-c_avg); % Eq. 14

%% Calculate minimum possible number of contacts, C_min

if ~alternatives
    
    A_R = @(x) redArea(r,h,x); % Eq. 15
    area_diff = @(x) A_R(x) - c_avg * bulk_area;
    initial_l = r + (2*c_avg - 1) * sqrt(h * (2*r - h));
    l_tilde = fzero(area_diff, initial_l); % Eq. 13
    % l_tilde: horizontal position of vertical line dividing the two colors
    
    C_min = h - r + sqrt(l_tilde * (2*r - l_tilde)); % Eq. 12
    
else
    [C_min,C_min_s,C_min_a] = separatingLength(r,h,c_avg);
end


%% Calculate modified number of actual contacts in row direction, C1

% Pixel shifting in the row direction
nzrows = arrayfun(@(ii) nonzeros(P(ii,:)), 1:H, 'uniformoutput', false);

% Counting contacts in each shifted row
C1 = 0;
for ii = 1:H
    C1 = C1 + find_noOfContacts(nzrows{ii},'col');
    % The last argument is 'col' because the elements of nzrows are column
    % vectors.
end

%% Calculate modified number of actual contacts in column direction, C2

% Pixel shifting in the column direction
nzcols = arrayfun(@(jj) nonzeros(P(:,jj)), 1:W, 'uniformoutput', false);

% Counting contacts in each shifted column
C2 = 0;
for jj = 1:W
    C2 = C2 + find_noOfContacts(nzcols{jj},'col');
end

%% Calculate modified mixing index

C = C1 + C2;
M = (C - C_min)/(C_max - C_min); % Eq. 11

if alternatives
    Ms = (C - C_min_s)/(C_max - C_min_s);
    Ma = (C - C_min_a)/(C_max - C_min_a);
end