function [M,vertex] = M_variance(P,winSize,vertex)

% Calculates the mixing index of an image using the variance method.

% Theodoros-Nestor Papapetrou
% Last edited: 06 Oct 2020

%% Input

% P: uint8 H x W matrix with values corresponding to colors:
% 1: red, 2: white, 0: background (black)
% P represents an image of the rotating drum with red and white particles

% winSize: integer array with one or two elements
% It defines the size of the rectangle-shaped window as [noOfRows noOfCols]
% Each element should be less than the corresponding size of P
% If winSize has one element, square-shaped windows are used

% vertex: integer array with two elements [rowNumber colNumber]
% It is the position of the top-left pixel of a window in the window grid,
% therefore it determines the position of the entire window grid on top of
% the image

%% Output

% M: the value of the mixing index, double scalar between 0 and 1

% vertex: the position of the top-left pixel of the first full window on
% the top left of the image

%% Preliminary calculations

[H,W] = size(P);

if numel(winSize) > 2
    winSize = winSize(1:2);
end

if numel(winSize) == 1
    winSize = [winSize winSize];
end

winSize = floor(winSize);

%% Calculate top left vertex

vertex = floor(vertex(1:2));
vertex = find_vertex(vertex,winSize);

%% Calculate average red concentration and maximum variance

N_rw = sum(sum(P ~= 0)); % total number of red and white pixels
N_r = sum(sum(P == 1)); % total number of red pixels
c_avg = N_r / N_rw; % average red concentration
s2max = c_avg * (1 - c_avg); % maximum variance

%% Add extra empty rows on top and columns to the left

% Extra rows and columns may need to be added, to complete windows at the
% edges of the image

extraLines = mod([1 1] - vertex, winSize);
He = extraLines(1); % number of extra black rows on top
We = extraLines(2); % number of extra black columns to the left

top_empty = zeros([He, We+W],'uint8');
left_empty = zeros([H We],'uint8');

P = [top_empty; left_empty P];

%% Calculate local weights (w) and local concentrations (c)

fun_r = @(block_struct) find_local_noOfRedPixels(block_struct.data);
fun_rw = @(block_struct) find_local_noOfColorPixels(block_struct.data);

N_r_local = blockproc(P, winSize, fun_r); % number of red pixels in
% each window
N_rw_local = blockproc(P, winSize, fun_rw); % number of color pixels in
% each window

w = N_rw_local / N_rw;

c = N_r_local;
for k = 1:numel(c)
    if c(k) > 0
        c(k) = c(k) / N_rw_local(k);
    % else, if c(k) == 0, there are no red pixels in the window, so the
    % local concentration is zero. This is true even for totally black
    % windows. These are not taken into account, as their local weight is
    % also zero.
    end
end

%% Calculate weighted variance and M

s2 = sum(sum(w.*(c - c_avg).^2));
M = 1 - s2./s2max;

end

function N_r = find_local_noOfRedPixels(window)

N_r = sum(sum(window == 1));

end

function N_rw = find_local_noOfColorPixels(window)

N_rw = sum(sum(window ~= 0));

end

function vertex_top_left = find_vertex(vertex,winSize)

vertex_top_left = mod(vertex - [1 1],winSize) + [1 1];

end