function [BWRed,BWWhite,params] = RGBVid2BWRedWhite(RGBVid,params)

% Segments each frame of an experimental color video, RGBVid, into its red,
% white and black regions, and creates two binary masks, BWRed and BWWhite,
% with the positions of the red and white pixels respectively.
% Morphological operations are not applied.

% Theodoros-Nestor Papapetrou
% Last edited: 08 Oct 2020

%% Input

% RGBVid: H x W x C x F uint8 array, color video where C is the number of
% color channels (expected to be 3) and F is the number of frames. RGBVid
% must be cropped, i.e. only the inside of the rotating drum must be
% visible.

% params: structure with the following fields:
%   thresRedHSV: array with 2 scalar elements between 0 and 1, the
%       thresholds used in the function createMaskRedHSV
%   thresRedRGB: scalar between 0 and 255, the threshold used in the
%       function createMaskRedRGB
%   thresWhiteHSV: array with 2 scalar elements between 0 and 1, the
%       thresholds used in the function createMaskWhiteHSV
% If some of these fields does not exist or does not contain enough values
% or if some values are out of range, the default threshold values of the 
% respective function are used and these are returned and saved in params.

%% Output

% BWRed: H x W x F logical array, with 1 values where RGBVid has red
% particles. This is matrix K in the paper.

% BWWhite: H x W x F logical array, with 1 values where RGBVid has white
% particles. This is matrix Q in the paper.

% params: same as the input, where the default thresholds have been added
% if necessary.

%% Initialization

vidSize = size(RGBVid);

% Check if RGBVid is a video (F > 1) or a single image (F = 1)
if numel(vidSize) == 3 % single image
    F = 1;
else % video
    F = vidSize(4);
end

BWRed = false([vidSize(1) vidSize(2) F]);
BWWhite = BWRed;

%% First frame

RGB = RGBVid(:,:,:,1);

if checkFieldname(params,'thresRedHSV',2,[0 1])
    thresRedHSV = params.thresRedHSV;
    [~,RGBRed] = createMaskRedHSV(RGB,thresRedHSV);
else
    [~,RGBRed,thresRedHSV] = createMaskRedHSV(RGB);
    params.thresRedHSV = thresRedHSV;
end

if checkFieldname(params,'thresRedRGB',1,[0 255])
    thresRedRGB = params.thresRedRGB;
    BW = createMaskRedRGB(RGBRed,thresRedRGB);
else
    [BW,~,thresRedRGB] = createMaskRedRGB(RGBRed);
    params.thresRedRGB = thresRedRGB;
end

BWRed(:,:,1) = BW;

if checkFieldname(params,'thresWhiteHSV',2,[0 1])
    thresWhiteHSV = params.thresWhiteHSV;
    BW = createMaskWhiteHSV(RGB,thresWhiteHSV);
else
    [BW,~,thresWhiteHSV] = createMaskWhiteHSV(RGB);
    params.thresWhiteHSV = thresWhiteHSV;
end

BWWhite(:,:,1) = BW;

%% Second to last frame

if F > 1
    for fr = 2:F

        RGB = RGBVid(:,:,:,fr);

        [~,RGBRed] = createMaskRedHSV(RGB,thresRedHSV);
        BW = createMaskRedRGB(RGBRed,thresRedRGB);
        BWRed(:,:,fr) = BW;

        BW = createMaskWhiteHSV(RGB,thresWhiteHSV);
        BWWhite(:,:,fr) = BW;

    end
end

end

function TF = checkFieldname(S,fieldname,N,fieldrange)

% Checks whether a given field in the structure S exists, whether it is
% numeric, whether it contains a minimum number of elements, N, and whether
% all elements 1-N are inside the given range. If all these are true, TF =
% 1, otherwise TF = 0.

if ~isfield(S,fieldname)
    TF = 0;
else
    fld = S.(fieldname);
    if ~isnumeric(fld)
        TF = 0;
    elseif numel(fld) < N
        TF = 0;
    else
        fld = fld(1:N);
        if any(fld < min(fieldrange))
            TF = 0;
        elseif any(fld > max(fieldrange))
            TF = 0;
        else
            TF = 1;
        end
    end 
end

end