function [Pvid,params] = RGBVid2P(RGBVid,params,track)

% Implements the automatic image preprocessing to each frame of a color
% video, RGB, segmenting it into its red, white and black regions, and
% creates the corresponding P matrices, using the parameters in the
% structure params.

% Theodoros-Nestor Papapetrou
% Last edited: 08 Oct 2020

%% Input

% RGBVid: H x W x C x F uint8 array, color video where C is the number of
% color channels (expected to be 3) and F is the number of frames. RGBVid
% must be cropped, i.e. only the inside of the rotating drum must be
% visible.

% params: structure with the following fields:
%   thresRedHSV: array with 2 scalar elements between 0 and 1, the
%       thresholds used in the function createMaskRedHSV
%   thresRedRGB: scalar between 0 and 255, the threshold used in the
%       function createMaskRedRGB
%   thresWhiteHSV: array with 2 scalar elements between 0 and 1, the
%       thresholds used in the function createMaskWhiteHSV
%   noOfMorphOps: nonnegative integer, the number of erosions and dilations
%   strelWidth: positive integer, width of the square structuring element
%       used in the erosions and dilations. It should be an odd number at
%       least 3; otherwise, it is rounded down to the closest odd.
% If some of these fields does not exist or does not contain enough values
% or if some values are out of range, the default values are used and these
% are returned and saved in params.

% track: nonnegative integer, indicates how often the current frame under
% process is displayed. If track equals 0 or is not given, nothing is
% displayed.

%% Output

% Pvid: H x W x F uint8 array with values corresponding to colors:
% 1: red, 2: white, 0: black (background)
% Each page of Pvid, Pvid(:,:,ii) represents an image of the rotating drum
% at frame ii with red and white particles

% params: same as the input, where the field values have been added or
% corrected if necessary.

%% Check track

if nargin < 3
    track = 0; % default value
else
    track = max(0,floor(track));
end

%% Create separate binary masks for red and white

if track
    disp('Creating binary masks...');
end
[BWRed,BWWhite,params] = RGBVid2BWRedWhite(RGBVid,params);
if track
    disp('Binary masks created.');
end

%% Combine binary masks into Pvid

if track
    disp('Merging binary masks...');
end
[Pvid,params] = BWRedWhite2P(BWRed,BWWhite,params,track);
if track
    disp('Binary masks merged.');
end