function [a_repose, X, Y, polyn, R2] = angleOfRepose(P)

% Calculates the average dynamic angle of repose, in degrees, for an image
% of the rotating drum, by approximating the free surface with a 1st order
% polynomial. The image should not have any noise above the free surface.

% Theodoros-Nestor Papapetrou
% Last edited: 06 Nov 2020

%% Input

% P: uint8 H x W matrix with values corresponding to colors:
% 1: red, 2: white, 0: background (black)
% P represents an image of the rotating drum with red and white particles

% Remark: for the calculations of this function, the actual values of the
% elements of P do not matter, as long as the background has the value 0
% and the foreground (granular particles) have nonzero values.

%% Output

% a_repose: the average angle of repose (scalar, in degrees). It can have a
% negative value if the drum rotates counterclockwise.

% X: the x-coordinates of the pixels belonging to the free surface

% Y: the y-coordinates of the pixels belonging to the free surface

% polyn = [s b]: the coefficients of the 1st-order polynomial approximation
% of the free surface (s: slope, b: abscissa; scalars)

% R2: the R^2 coefficient of the approximation (scalar between 0 and 1)

%% Extract the coordinates of the top (not necessarily free) surface
[Xtop, Ytop] = topSurface(P);

%% Extract the coordinates of the free surface
[x_left, x_right] = topSurfacePointEdges(P);
Xfree = (Xtop >= x_left) & (Xtop <= x_right);
X = Xtop(Xfree);
Y = Ytop(Xfree);

%% Linear regression
polyn = polyfit(X,Y,1);
slope = polyn(1);
a_repose = atand(slope);

%% R^2 of linear regression
%https://de.mathworks.com/help/matlab/data_analysis/linear-regression.html
yfit = polyval(polyn,X);
yresid = Y - yfit;
SSresid = sum(yresid.^2);
SStotal = (length(Y)-1) * var(Y);
R2 = 1 - SSresid/SStotal;