function [stat_str, new_meas_str] = calculateParticleDiameter(meas_str)

% Calculates the statistics related to the calculation of the diameter of
% the particles, in terms of pixels, from an experimental image.

% Theodoros-Nestor Papapetrou
% Last edited: 07 Oct 2020

%% Input

% meas_str: structure array with measurement data. It must contain the
% fields:
    % color: the color of the measured particle. Possible values: 'R'
        % (red), 'W' (white)
    % x1: the column number of the leftmost pixel of the particle (integer)
    % x2: the column number of the rightmost pixel of the particle (integer)
    % y1: the row number of the top pixel of the particle (integer)
    % y2: the row number of the bottom pixel of the particle (integer)
% It is expected that x1 < x2 and y1 < y2.        

%% Output

% stat_str: structure array with statistics derived from the measurement
% data. It contains the fields:
    % subset: indicates which subset of the measurement data is used for
        % the calculations. It takes the values:
        % {'W Dx';'W Dy';'R Dx';'R Dy';'W';'R';'Dx';'Dy';'all'}
    % mean: the average particle diameter, as calculated using the
        % respective data (double scalar, value in pixels)
    % std: the standard deviation of the particle diameter, as calculated
        % using the respective data (double scalar, value in pixels)
        
% new_meas_str: the original meas_str, with two new fields:
    % Dx = |x2 - x1|; Dy = |y2 - y1|;
    
%% Calculation of Dx, Dy, number of red/white particles

N = numel(meas_str); % total number of particles

N_R = 0; % number of red particles
N_W = 0; % number of white particles
N_o = 0; % number of particles neither red nor white
SDxR = 0; % sum of Dx of red particles
SDx2R = 0; % sum of Dx^2 of red particles
SDyR = 0; % sum of Dy of red particles
SDy2R = 0; % sum of Dy^2 of red particles
SDxW = 0; % sum of Dx of white particles
SDx2W = 0; % sum of Dx^2 of white particles
SDyW = 0; % sum of Dy of white particles
SDy2W = 0; % sum of Dy^2 of white particles
new_meas_str = meas_str;

for ii = 1:N
    if any(strcmp(meas_str(ii).color,{'R','W'}))
        x1 = meas_str(ii).x1;
        x2 = meas_str(ii).x2;
        y1 = meas_str(ii).y1;
        y2 = meas_str(ii).y2;
        Dx = abs(x2 - x1);
        Dy = abs(y2 - y1);
        new_meas_str(ii).Dx = Dx;
        new_meas_str(ii).Dy = Dy;
        
        if strcmp(meas_str(ii).color,'R') % red particle
            N_R = N_R + 1;
            SDxR = SDxR + Dx;
            SDx2R = SDx2R + Dx^2;
            SDyR = SDyR + Dy;
            SDy2R = SDy2R + Dy^2;

        else % white particle
            N_W = N_W + 1;
            SDxW = SDxW + Dx;
            SDx2W = SDx2W + Dx^2;
            SDyW = SDyW + Dy;
            SDy2W = SDy2W + Dy^2;
        
        end
    else
        N_o = N_o + 1;
    end
end

if N_o > 0
    warning([num2str(N_o),' particles were neither red nor white.']);
end

%% Calculation of statistics

stat_str(1).subset = 'R Dx';
mRx = SDxR/N_R; % average diameter for R Dx
sRx = calc_std(N_R,mRx,SDx2R); % standard deviation for R Dx
stat_str(1).mean = mRx;
stat_str(1).std = sRx;

stat_str(2).subset = 'R Dy';
mRy = SDyR/N_R; % average diameter for R Dy
sRy = calc_std(N_R,mRy,SDy2R); % standard deviation for R Dy
stat_str(2).mean = mRy;
stat_str(2).std = sRy;

stat_str(3).subset = 'W Dx';
mWx = SDxW/N_W; % average diameter for W Dx
sWx = calc_std(N_W,mWx,SDx2W); % standard deviation for W Dx
stat_str(3).mean = mWx;
stat_str(3).std = sWx;

stat_str(4).subset = 'W Dy';
mWy = SDyW/N_W; % average diameter for W Dy
sWy = calc_std(N_W,mWy,SDy2W); % standard deviation for W Dy
stat_str(4).mean = mWy;
stat_str(4).std = sWy;

stat_str(5).subset = 'R';
mR = (SDxR + SDyR)/(2*N_R); % average diameter for R
sR = calc_std(2*N_R,mR,SDx2R+SDy2R); % standard deviation for R
stat_str(5).mean = mR;
stat_str(5).std = sR;

stat_str(6).subset = 'W';
mW = (SDxW + SDyW)/(2*N_W); % average diameter for W
sW = calc_std(2*N_W,mW,SDx2W+SDy2W); % standard deviation for W
stat_str(6).mean = mW;
stat_str(6).std = sW;

stat_str(7).subset = 'Dx';
mx = (SDxR + SDxW)/(N_R + N_W); % average diameter for Dx
sx = calc_std(N_R+N_W,mx,SDx2R+SDx2W); % standard deviation for Dx
stat_str(7).mean = mx;
stat_str(7).std = sx;

stat_str(8).subset = 'Dy';
my = (SDyR + SDyW)/(N_R + N_W); % average diameter for Dy
sy = calc_std(N_R+N_W,my,SDy2R+SDy2W); % standard deviation for Dy
stat_str(8).mean = my;
stat_str(8).std = sy;

stat_str(9).subset = 'all';
m = (SDxR + SDyR + SDxW + SDyW)/(2*(N_R + N_W)); % average diameter
s = calc_std(2*(N_R+N_W),m,SDx2R+SDy2R+SDx2W+SDy2W); % standard deviation
stat_str(9).mean = m;
stat_str(9).std = s;

end

function stdev = calc_std(N,m,Sx2)

% Calculates the standard deviation of a set of data.

%% Input

% N: number of data points (integer)
% m: the mean of the data (double scalar)
% Sx2: the sum of squares of the data (double scalar)

%% Output

% stdev: the standard deviation of the data (double scalar). The Bessel
% correction (N-1) is used in the denominator.

%% Calculation

stdev = sqrt((Sx2-N*m^2)/(N-1));

end

