% cameraProcessing1

% The video from the camera must have already been imported using:
% vid = readAllVideo(video_path);
% The first frame of interest (e.g. right before the rotation starts)
% has to be found, using e.g.:
% implay(vid);
% The position of a black pixel in the upper half of the drum must have
% been found, using e.g.:
% imshow(vid(:,:,:,??)); impixelinfo;

% Theodoros-Nestor Papapetrou
% Last edited: 12 Jul 2022

%% Initialization

clearvars -except vid

%% Create results folder

if ~isfolder('results')
    mkdir results
else % check if the folder is empty
    if numel(dir('results')) > 2 % not empty
        disp('The ''results'' folder contains data.');        
        overwrite = input('Are you sure you want to delete them? (1: yes, 0: no)? ');
        overwrite = logical(overwrite);
        if overwrite
            rmdir results s
            mkdir results
        else
            error('Move the data from the ''results'' folder and try again.');
        end
    end
end

clear overwrite

%% Read the operating conditions

accept = 0;
while ~accept

    % Date of measurement

    year = input('Give the year of the measurement: ');
    month = input('Give the month of the measurement (1-12): ');
    day = input('Give the day of the measurement (1-31): ');
    experimentalConditions.date = datetime(year,month,day);
    clear year month day

    % Nominal filling degree

    f = input('Give the volume filling degree (0-1): ');
    experimentalConditions.fillingDegree = f;

    % Mass of the particles

    mG = input('Give the mass of the glass particles (in g): ');
    mGerr = input('Give the error of the mass of the glass particles (in g): ');
    mPP = input('Give the mass of the polypropylene particles (in g): ');
    mPPerr = input('Give the error of the mass of the polypropylene particles (in g): ');
    experimentalConditions.glassMass_g.value = mG;
    experimentalConditions.glassMass_g.error = mGerr;
    experimentalConditions.polypropyleneMass_g.value = mPP;
    experimentalConditions.polypropyleneMass_g.error = mPPerr;

    % Rotational speed

    w_rpm = input('Give the rotational speed of the drum (in rpm): ');
    w_Hz = w_rpm/60;
    experimentalConditions.rotationalSpeed_rpm = w_rpm;
    experimentalConditions.rotationalSpeed_Hz = w_Hz;

    % Number of rotations

    U = input('Give the number of rotations of the drum: ');
    experimentalConditions.noOfRotations = U;

    % Acceleration and deceleration time

    t_acc = input('Give the acceleration time of the drum (in s): ');
    t_dec = input('Give the deceleration time of the drum (in s): ');
    experimentalConditions.accelerationTime_s = t_acc;
    experimentalConditions.decelerationTime_s = t_dec;
    
    % Camera framerate
    
    frate = input('Give the camera frame rate (in frames/s): ');
    experimentalConditions.framerate_fps = frate;

    disp(experimentalConditions)
    glassMass_g = experimentalConditions.glassMass_g;
    polypropyleneMass_g = experimentalConditions.polypropyleneMass_g;
    disp(glassMass_g)
    disp(polypropyleneMass_g)

    accept = logical(input('Check the values you gave. Are they correct (1: yes, 0: no)? '));

end

clear accept

%% Read starting frame number
% This frame corresponds to time = 0

startingFrameNumber = input('Give the number of the first frame: ');

%% Create arrays for time, frames, rotations

noOfFrames = size(vid,4);

% Frame array
% The values indicate the camera frame number. The starting frame is 0.
frame = [1:noOfFrames] - startingFrameNumber;

% Time array
% The values indicate the seconds passed since the starting frame.
time = frame / frate;

% Rotations array
% The values indicate the number of rotations since the starting frame.
rotationsNaive = time * w_Hz;
totalRotationTime = U / w_Hz;
w = zeros(1,noOfFrames); % exact rotational speed
r = zeros(1,noOfFrames); % exact rotations
dt = 1 / frate;
for ii = 1:noOfFrames
    if time(ii) <= 0 % before the rotation has begun
        w(ii) = 0;
        r(ii) = 0;
    elseif time(ii) <= t_acc % acceleration
        w(ii) = w_Hz / t_acc * time(ii);
        r(ii) = r(ii-1) + (w(ii) + w(ii-1))*dt/2;
    elseif time(ii) <= totalRotationTime - t_dec % constant rotational speed
        w(ii) = w_Hz;
        if time(ii) - t_acc >= dt
            r(ii) = r(ii-1) + w_Hz*dt;
        else
            dt2 = time(ii) - t_acc;
            dt1 = dt - dt2;
            r(ii) = r(ii-1) + (w(ii) + w(ii-1))*dt1/2 + w_Hz*dt2;
        end
    elseif time(ii) <= totalRotationTime % deceleration
        w(ii) = w_Hz / t_dec * (totalRotationTime - time(ii));
        if time(ii) - (totalRotationTime - t_dec) >= dt
            r(ii) = r(ii-1) + (w(ii) + w(ii-1))*dt/2;
        else
            dt2 = time(ii) - (totalRotationTime - t_dec);
            dt1 = dt - dt2;
            r(ii) = r(ii-1) + w_Hz*dt1 + (w(ii) + w(ii-1))*dt2/2;
        end
    else % after the rotation has stopped
        w(ii) = 0;
        if time(ii) - totalRotationTime >= dt
            r(ii) = r(ii-1);
        else
            dt2 = time(ii) - totalRotationTime;
            dt1 = dt - dt2;
            r(ii) = r(ii-1) + (w(ii) + w(ii-1))*dt1/2;
        end
    end
end
rotationalSpeedExact_Hz = w;
rotationalSpeedExact_rpm = w*60;
rotationsExact = r;
stoppingFrameNumber = find(w == max(w),1,'last');

%% Mask video

disp('Creating circular mask...');
row = input('Give the row of the black pixel, in the upper half of the drum: ');
col = input('Give the column of the black pixel, in the upper half of the drum: ');
pixel = [row col];
maskingParameters = detectDrumCircleVideo(vid, pixel, 20);
disp('Circular mask created');
disp('Masking the video...');
vidMasked = maskVideoWithCircle(vid,maskingParameters);
disp('Masked video created.');
disp('Saving masked video...');
writeVideo2Path(vidMasked, 'results\masked.avi', frate);
disp('Masked video saved.');
implay(vidMasked);

clear year month day f mG mGerr mPP mPPerr w_rpm w_Hz U t_acc t_dec
clear frate glassMass_g polypropyleneMass_g w r dt dt1 dt2 ii row col pixel

save('results\data.mat');
