function Par_rowcol = detectDrumCircleVideo(vid, pixel, track)

% Calculates the position and radius of a rotating drum based on all frames
% of an experimental RGB video. The position and radius for each frame is
% calculated using the function:
% Par_rowcol = detectDrumCircle_imfill(RGBimg,pixel)
% The median values for the coordinates of the drum center are used. For
% the drum radius, 3 standard deviations are subtracted from the median
% value. This is done to undo the effect of detectDrumCircle_imfill, which
% overestimates the radius.

% Theodoros Nestor Papapetrou
% Last edited: 24 Nov 2020

%% Input

% vid: H x W x 3 x F uint8 matrix, the experimental RGB video of the
% rotating drum

% pixel: 1 x 2 integer, pixel coordinates (row, column) that determine the
% region to be filled. The pixel must be inside the drum and above the
% particle bed. If it is not given or if an empty array is given, the
% default value [500 300] is used.

% track: nonnegative integer value. It is used for keeping track the
% progress of the function calculation. The user is informed every (track)
% frames about the frame currently processed. If track is 0 (default), the
% user is never informed.

%% Output

% Par_rowcol: 1 x 3 double matrix, the values of the drum circle
% properties, across all frames of vid, in the order:
%   column of the drum center (median)
%   row of the drum center (median)
%   inner radius of the drum (median minus 3 standard deviations, in
%       pixels)

%% Initialization

if nargin < 3
    
    track = 0; % default value
    if nargin < 2
        pixel = [];
    end
    
else
    
    track = ceil(track(1)); % ensure track is an integer scalar
    track = max(track, 0); % ensure track is nonnegative
    if ~isempty(pixel)
        pixel = floor(pixel(1:2));
    end
    
end

F = size(vid,4);
Par_matrix = zeros(F,3);
if track > 0
    strF = num2str(F);
end

%% Calculation for each frame

for fr = 1:F
    RGBimg = vid(:,:,:,fr);
    Par_matrix(fr,:) = detectDrumCircle_imfill(RGBimg,pixel);
    if mod(fr, track) == 0
        strfr = num2str(fr);
        msg = ['Current frame: ',strfr,'/',strF];
        disp(msg);
    end
end

%% Omit NaN values and outliers and calculate circle parameters

Par_rowcol = zeros(1,3);
values_std = zeros(1,3);
for ii = 1:3
    values = Par_matrix(:,ii); % all calculated rows, columns or radii
    values = rmoutliers(values);
    Par_rowcol(ii) = median(values,'omitnan');
    values_std(ii) = std(values,'omitnan');
end

%% Radius correction

Par_rowcol(3) = Par_rowcol(3) - 3*max(values_std);