function [Par_rowcol,vid_radius,D_sum_over_area] = detectDrumCircle_imfill(RGBimg,pixel,show)

% Automatically detects the drum circle from an experimental RGB image.
% The following code is used for the fitting of the circle:

% Nikolai Chernov (2020). Circle Fit (Pratt method)
% (https://www.mathworks.com/matlabcentral/fileexchange/22643-circle-fit-pratt-method),
% MATLAB Central File Exchange. Retrieved August 15, 2019.

% The detected drum radius is usually bigger than the real one, because of
% the morphological operations used. A sensitivity analysis is performed on
% the radius, to find a better estimation.

% Theodoros-Nestor Papapetrou
% Last edited: 26 Oct 2020

%% Input

% RGBimg: H x W x 3 uint8 matrix, the experimental image

% pixel: 1 x 2 integer, pixel coordinates (row, column) that determine the
% region to be filled. The pixel must be inside the drum and above the
% particle bed. If it is not given or if an empty array is given, the
% default value [500 300] is used.

% show: logical scalar, determines if the intermediate BW images and the
% plots of the sensitivity analysis are shown (1) or not (0, default)

%% Output

% Par_rowcol = [x y r]: the parameters of the circle approximation of the
% drum (x, y: coordinates of the center, r: radius; scalars, all in pixels)

% vid_radius: H x W x 3 x 50 uint8 matrix, the video of the sensitivity
% analysis. It is a video with 50 frames, where the radius is increased
% incrementally from 80% to 100% of the estimated value

% D_sum_over_area: 1 x 50 double matrix, the increment in the total
% brightness of the image (calculated as the sum of the pixel values in the
% three color channels) for each increment of radius in the sensitivity
% analysis. The correct value of the radius corresponds to the point before
% the values of D_sum_over_area increase sharply

%% Parameters

pixel_default = [500 300];
edge_strel_radius = 2; % radius of disk structuring element for the edge
% widening
dark_strel_radius = 20; % radius of disk structuring element for the
% darkening

%% Initialization

H = size(RGBimg,1);
W = size(RGBimg,2);

if nargin < 3
    
    show = 0; % default value
    
    if nargin < 2
        pixel = pixel_default;
    else % nargin == 2
        if isempty(pixel)
            pixel = pixel_default;
        else
            pixel = floor(pixel(1:2));
        end
    end
    
else % nargin == 3
    
    show = logical(show(1));
    
    if isempty(pixel)
        pixel = pixel_default;
    else
        pixel = floor(pixel(1:2));
    end
    
end

%% Edge detection

R = RGBimg(:,:,1); % red channel of RGBimg
R = edge(R,'Canny');

if show
    figure
    subplot(2,3,1)
    imshow(R);
end

%% Edge widening

R = ~imerode(~R,strel('disk',edge_strel_radius));

if show
    subplot(2,3,2)
    imshow(R);
end

%% Filling

% Vary the chosen pixel a bit in the horizontal and vertical direction to
% prevent problems in case it is a noise pixel
for cc = pixel(2) - 10: pixel(2) + 10
    R = imfill(R, [pixel(1) cc]);
end
for rr = pixel(1) - 10: pixel(1) + 10
    R = imfill(R, [rr pixel(2)]);
end

if show
    subplot(2,3,3)
    imshow(R);
end

%% Darkening

% Darken the rest of the big white region inside the drum (above the
% particles) by eroding and dilating with a sufficiently big circle
strel_dark = strel('disk',dark_strel_radius);
R = imdilate(imerode(R,strel_dark),strel_dark);

if show
    subplot(2,3,4)
    imshow(R);
end

%% Keep biggest area

% Keep only the biggest white component (for some images, strel_dark may
% fit in some small areas of the background too)
CC = bwconncomp(R);
numPixels = cellfun(@numel,CC.PixelIdxList);
[~,idx] = max(numPixels);
R = false(size(R));
R(CC.PixelIdxList{idx}) = 1;

if show
    subplot(2,3,5)
    imshow(R);
end

%% Detect top surface

TopCols = zeros(W,1);
TopRows = zeros(W,1);
k = 0;
for cc = 1:W
    Rcol = R(:,cc);
    if sum(Rcol) > 0
        k = k + 1;
        TopCols(k) = cc;
        TopRows(k) = find(Rcol == 1,1);
    end
end
TopCols = TopCols(1:k);
TopRows = TopRows(1:k);

%% Convert from row-col to x-y and fit the circle

Xtop = imcols2x(TopCols);
Ytop = imrows2y(TopRows,H);
XYtop = [Xtop Ytop];

Par_xy = CircleFitByPratt(XYtop);

%% Convert center back to row-col

row_center = y2imrows(Par_xy(2),H);
col_center = x2imcols(Par_xy(1));
% Par_rowcol = [col_center row_center Par_xy(3)];
drum_radius = Par_xy(3);
Par_rowcol = [col_center row_center drum_radius];

%% Mask image

if show
    RGBMasked = maskVideoWithCircle(RGBimg,Par_rowcol);
    subplot(2,3,6)
    imshow(RGBMasked);
end

%% Radius sensitivity analysis

if nargout > 1

    noOfPoints = 50;
    r_calc = drum_radius*linspace(0.8,1,noOfPoints);
    dr = r_calc(2) - r_calc(1); % step in the variation of the radius
    rr = r_calc(1) - dr; % current drum radius
    D_sum_over_area = zeros(1,noOfPoints);
    Par_rowcol_temp = [col_center row_center rr];
    RGBMasked = maskVideoWithCircle(RGBimg,Par_rowcol_temp);
    drum_area = pi*rr^2;
    drum_sum = sum(RGBMasked,'all');
    vid_radius = zeros(H,W,3,noOfPoints,'uint8');

    for ii = 1:noOfPoints

        rr = r_calc(ii);
        Par_rowcol_temp = [col_center row_center rr];
        RGBMasked = maskVideoWithCircle(RGBimg,Par_rowcol_temp);
        vid_radius(:,:,:,ii) = RGBMasked;

        new_drum_area = pi*rr^2;
        new_drum_sum = sum(RGBMasked,'all');
        D_drum_area = new_drum_area - drum_area;
        D_drum_sum = new_drum_sum - drum_sum;
        drum_area = new_drum_area;
        drum_sum = new_drum_sum;
        D_sum_over_area(ii) = D_drum_sum / D_drum_area;

    end

    % 1st derivative

    r_calc1 = zeros(1,noOfPoints-1);
    y1 = r_calc1;

    for ii = 1:noOfPoints-1

        r_calc1(ii) = (r_calc(ii+1) + r_calc(ii))/2;
        y1(ii) = (D_sum_over_area(ii+1) - D_sum_over_area(ii))/dr;

    end

    % 2nd derivative

    r_calc2 = zeros(1,noOfPoints-2);
    y2 = r_calc2;

    for ii = 1:noOfPoints-2

        r_calc2(ii) = (r_calc1(ii+1) + r_calc1(ii))/2;
        y2(ii) = (y1(ii+1) - y1(ii))/dr;

    end

    if show
        figure
        subplot(1,3,1)
        scatter(r_calc, D_sum_over_area)
        subplot(1,3,2)
        scatter(r_calc1, y1)
        subplot(1,3,3)
        scatter(r_calc2, y2)
    end

end