function [f, polyn, drum_circle, h, phi, gran_area] = fillingDegree1(P)

% Calculates the filling degree and the granular area of an image by
% approximating the free surface with a 1st order polynomial and fitting a
% circle to the bottom drum surface.

% Theodoros-Nestor Papapetrou
% Last edited: 06 Nov 2020

%% Input

% P: uint8 H x W matrix with values corresponding to colors:
% 1: red, 2: white, 0: background (black)
% P represents an image of the rotating drum with red and white particles

%% Output

% f: area filling degree - the fraction of the circular drum area occupied
% by particles, double scalar between 0 and 1

% polyn = [s b]: the coefficients of the 1st-order polynomial approximation
% of the free surface (s: slope, b: abscissa; scalars)

% drum_circle = [x y r]: the parameters of the circle approximation of the
% drum (x, y: coordinates of the center, r: radius; scalars, all in pixels)

% h: average height of the granular bed (scalar, in pixels)

% phi: filling angle, defined at the drum center (scalar, in radians)

% gran_area: the granular area enclosed by all particles (scalar, in
% pixels)

%% Fit 1st order polynomial to top surface
[~, ~, ~, polyn] = angleOfRepose(P);
s = polyn(1); % slope
a = polyn(2); % abscissa

%% Find drum circle
drum_circle = drumCircle(P);
x0 = drum_circle(1); % center x-coordinate
y0 = drum_circle(2); % center y-coordinate
r = drum_circle(3); % circle radius

%% Final calculations
dist = s*x0 - y0 + a/sqrt(s^2 + 1); % signed distance of drum center to top surface
% dist < 0 if the center lies below the top surface
% dist > 0 if the center lies above the top surface

h = r + dist; % average height of bed, if tilted horizontally

phi = 2*acos(1 - h/r); % occupied angle; the central angle of the bed

f = (phi - sin(phi))/(2*pi);

gran_area = f*pi*r*r;
