function N = replaceMarked(M,mark,alt,width,method)

% Replaces the value of each of the marked elements of matrix M with the
% most appropriate value among the alternatives. The new value is chosen by
% using the square neighborhood of each element and a given method.

% Theodoros-Nestor Papapetrou
% Last edited: 07 Oct 2020

%% Inputs

% M: H x W uint8 matrix. It shoul with (possible) values:
% 0-2: normal values (black - red - white), plus an extra marker value for
% the elements that need to change value

% mark: the marker value, integer scalar. All elements of M equal to mark
% are replaced with one of the values in alt. All elements of M that have a
% value different than mark remain unaltered

% alt: uint8 array, the possible alternative values for the marked elements
% of M

% width: odd uint8 scalar, the width of the square neighborhood used. If it
% is not an odd integer, it gets rounded down to the nearest odd integer
% or rounded up to 3.

% method: the method used to choose the appropriate value among the
% alternatives, string with possible values: 'count', 'distance'.

% With the 'count' method, the elements of each alternative in the
% neighborhood are counted and the value with most elements is chosen.
% This is the method used in the paper.
% With the 'distance' method, each element's contribution is weighed by the
% inverse of its distance to the center of the neighborhood, i.e. the
% marked element under consideration.
% In either method, if there is no result, i.e. if the contribution of two
% or more alternatives is equal, the neighbourhood is enlarged and the
% process repeated.
% The default method is 'count'.

%% Output:

% N: H x W uint8 matrix, the matrix M with all marked elements having one
% of the alternative values

%% Initial calculations

% Check the method
if nargin < 5
    method = 'count';
elseif strncmpi(method,'c',1)
    method = 'count';
elseif strncmpi(method,'d',1)
    method = 'distance';
else
    error('Invalid method.')
end

% Reshape the alternatives
alt = reshape(alt,[numel(alt) 1]);

% Fix the neighborhood width
if width <= 3
    width = 3;
else % rounded down to the nearest odd integer
    width = floor(width);
    width = width + mod(width,2) - 1;
end

%% Map the values to the integers

% nfilter uses zero padding. Therefore, to avoid extra elements with a
% possible alternative value of zero to appear at the edges, the marker
% value is mapped to 0, the alternative values to the integers from 1 to
% numel(alt) and finally the rest of the values of M to the integers after
% that.

% Find the rest of the values in M
rest = unique(M);
rest = setdiff(rest,[mark; alt]);

% Mapping
% map is a vector with 2 columns. In the 1st column are the original
% values, in the 2nd column the corresponding new values 0, 1, ...
orig = [mark alt' rest'];
map = [orig' [0:numel(orig)-1]'];

Mmap = mapping(M,map(:,1),map(:,2));

%% Replace the marked elements (now with value 0)

% Filtering, with progressively larger width
Nmap = Mmap; % The output matrix, N, with the mapped values
w = width - 2;

while nnz(Nmap == 0) > 0
    
    w = w + 2;
    
    % Calculate the position of the center of the neighborhood
    center = floor(([w w]+1)/2);
    % from MATLAB documentation; for odd size(neigh) gives the geometrical
    % center
    
    % Calculate the coefficients for each method
    coef = ones([w w]); % if method is 'count'
    
    if isequal(method,'distance')
        for j = 1:w
            for i = 1:w
                coef(i,j) = 1 / (sum(([i j] - center).^2));
            end
        end
    end
    
    fun = @(neigh) chooseAlternative(neigh,numel(alt),center,coef);
    
    Nmap = nlfilter(Nmap,[w w],fun);
    
end

%% Inverse the mapping to retrieve the original values

N = mapping(Nmap,map(:,2),map(:,1));

end

function x = chooseAlternative(neigh,noOfAlts,center,coef)

% Chooses the appropriate value among the alternatives for the center of
% the given neighbourhood (neigh) using the given method.
% The alternatives must already be mapped to the integers [1:noOfAlts].
% The value should change if and only if the center has value 0.


% Check if the value of the center should change
center_value = neigh(center(1),center(2));
if center_value ~= 0 % The value must not change
    x = center_value;
else % The value must change
    
    % Calculate the contribution of every alternative.
    contrib = zeros(noOfAlts,1);
    
    for element = 1:numel(neigh)
        
        [~,value] = ismember(neigh(element),1:noOfAlts);
        if value > 0
            contrib(value) = contrib(value) + coef(element);
        end
        
    end
    
    % Decide if there is a clear result, otherwise return 0
    tol = 1.e-6;
    [best,ind] = maxk(contrib,2);
    
    if best(1) - best(2) > tol
        x = ind(1);
    else
        x = 0;
    end
    
end

end

function matrix_new = mapping(matrix,inputs,outputs)

% Values in matrix equal to any of the values in inputs get mapped to the
% corresponding output. Values in matrix that do not appear in inputs
% remain unaltered.

% inputs and outputs must have the same number of elements

matrix_new = matrix;
for i = 1:numel(inputs)
    matrix_new(matrix == inputs(i)) = outputs(i);
end

end