function [L_vert,L_line,L_arc,params] = separatingLength(R,h,c)

% Calculates the length of the separating curve between the right and left
% regions of a rotating drum of radius R and uniform filling height h, in
% the artificial limiting distribution where the two granular species are
% perfectly segregated in a left and a right region. The right region
% contains a fraction (c) at most half of the total area under the free
% surface.
% Three scenarios are considered: (a) the two regions are separated by a
% straight vertical line (perpendicular to the horizontal free surface),
% (b) the two regions are separated by a straight line of optimal angle,
% such that its length is minimized, (c) the two regions are separated by a
% circular arc, whose center lies on the intersection of the drum circle
% and the free granular surface, in the right region.
% In all cases, the separating curve connects the horizontal free surface
% to the drum circle.
% The calculations use the function:
% [A_right,len,c_right,A_perc] = rightArea(R,h,shape,params)

% Theodoros-Nestor Papapetrou
% Last edited: 02 Oct 2020

%% Input

% R: the radius of the rotating drum (positive double scalar)
% h: the filling height of the drum; positive double scalar, 0<h<=R
% c: the overall granular concentration of the species in the right area;
    % equivalently, the ratio of the right area to the total area of the
    % granular bed, i.e. the area under the free surface; double scalar,
    % 0<c<=0.5.

%% Output

% L_vert: length of the vertical separating line (positive double scalar)
% L_line: length of the optimal straight separating line (positive double
    % scalar)
% L_arc: length of the separating circular arc with fixed center
    % (positive double scalar)
% params: geometric parameters of the system and the separating curve for
    % all cases. Structure with four fields:
    % input: input data of this function. Structure with fields: R, h, c
    % vert: parameters of the vertical separating line. Structure with
        % fields:
        % x: x-coordinate of the vertical separating line (double scalar)
    % line: parameters of the optimal straight separating line. Structure
        % with fields:
        % x: x-coordinate of the intersection of the horizontal free
            % surface and the separating line (double scalar)
        % eta: the angle between the separating line and negative y-axis
            % (double scalar, 0<=eta<pi/2 in radians)
    % arc: parameters of the circular arc separating curve with fixed
        % center. Structure with fields:
        % r: radius of the circular arc (double scalar)
    % In the fields vert, line and arc, there are also the fields:
        % L: the length of the corresponding separating curve (double
            % scalar)
        % c: the concentration of the right area, as calculated using the
            % solution parameters (double scalar)
        % dcperc: the error of the calculated concentration from the given
            % one, as a percentage of the given one (double scalar)

%% Check input ranges

% if R <= 0
%     error('The drum radius R must be positive.')
% end
% 
% if abs(h-R/2) > R/2
%     error('The filling height h must be positive and less than the drum diameter.')
% end
% 
% if abs(c-0.25) > 0.25
%     error('The concentration c needs to be strictly between 0 and 1.')
% end

%% Initial calculations

y0 = h - R; % y-coordinate of the free surface
xA = sqrt(R^2 - y0^2); % x-coordinate of the right intersection of the free
                       % surface with the drum circle (point A)
% x_init = 2*xA*(c - 0.5); % initial x-coordinate for numerical solver
x_init = [-xA xA];
% phi = 2*acos(-y0/R); % angle of the total filled area arc
% A_bed = R^2*(phi - sin(phi))/2; % area of the granular bed

params.input.R = R;
params.input.h = h;
params.input.c = c;

%% Calculate L_vert

c_vert = @(x) concentration(R,h,'l',[x, 0]); % right region concentration
                                             % with vertical line
Dc_vert = @(x) c_vert(x) - c; % concentration difference (must equal 0)
x_vert = fzero(Dc_vert, x_init);
L_vert = curveLength(R,h,'l',[x_vert, 0]);

params.vert.x = x_vert;
c_calc = c_vert(x_vert);
params.vert.c = c_calc;
params.vert.dcperc = errper(c,c_calc);
params.vert.L = L_vert;

%% Calculate L_line

% c_line = @(xeta) concentration(R,h,'l',xeta); % right region concentration
%                                               % with optimal line
% Dc_line = @(xeta) c_line(xeta) - c; % concentration difference (must equal 0)
linefun = @(xeta) curveLength(R,h,'l',xeta); % line length, to be minimized
% Lsconstr = @(eeta) curveLength(R,h,xs(R,h,eeta,c),'l',eeta); % length
    % function constrained, so that x is such that the concentration is c
nlcon = @(xeta) line_nlcon(R,h,c,xeta); % sets the concentration equal to c
options = optimoptions('fmincon','display','none');
[xeta,L_line] = fmincon(linefun,[xA/2 pi()/4],[],[],[],[],[0 0],[xA pi()/2],nlcon,options);

params.line.x = xeta(1);
params.line.eta = xeta(2);
c_calc = concentration(R,h,'l',xeta);
params.line.c = c_calc;
params.line.dcperc = errper(c,c_calc);
params.line.L = L_line;

%% Calculate L_arc

% Aa = @(x) rightArea(R,h,x,'a',0); % right region area with vertical line
% DAa = @(x) Aa(x) - c * A_bed; % area difference between function and
%                               % true value
% xa = fzero(DAa, x_init);
% [~,L_arc_fix] = rightArea(R,h,xa,'a',0);

c_arc = @(r) concentration(R,h,'a',[xA, y0, r]); % right region
                                    % concentration with arc
Dc_arc = @(r) c_arc(r) - c; % concentration difference (must equal 0)
% r_arc_fix = fzero(Dc_arc_fix, [0 2*xA]);
r_arc = fzero(Dc_arc, [0 1.9*xA]);
L_arc = curveLength(R,h,'a',[xA, y0, r_arc]);

params.arc.r = r_arc;
c_calc = concentration(R,h,'a',[xA, y0, r_arc]);
params.arc.c = c_calc;
params.arc.dcperc = errper(c,c_calc);
params.arc.L = L_arc;

end

function [cneq,Dc_line] = line_nlcon(R,h,c,xeta)

% Nonlinear constraints for solving the optimization problem of case (b)
% (optimal line)

cneq = [];
c_line = concentration(R,h,'l',xeta); % right region concentration with
                                      % optimal line
Dc_line = c_line - c; % concentration difference (must equal 0)

end

function len = curveLength(R,h,shape,params)

% Returns the len output from the function:
% [A_right,len,c_right,A_perc] = rightArea(R,h,shape,params)

[~,len] = rightArea(R,h,shape,params);

end

function conc = concentration(R,h,shape,params)

% Returns the c_right output from the function:
% [A_right,len,c_right,A_perc] = rightArea(R,h,shape,params)

[~,~,conc] = rightArea(R,h,shape,params);

end

function dcperc = errper(c_real,c_calc)

% Calculates the error of c_calc as a percentage of c_real

dcperc = (c_calc - c_real)/c_real*100;

end
