function [x_left, x_right, y_left, y_right, RR] = topSurfacePointEdges(P,d_p)

% Calculates the positions of the left and right edge points of the top
% free surface of a rotating drum. The output coordinates are given as x
% and y coordinates (not image columns / rows).
% Points in the top surface more left than the left edge point and more
% right than the right edge point belong to the drum circle and not to the
% free surface.
% A point is considered to belong to the top free surface (and not the drum
% circle surface) if its distance from the drum circle is more than a
% particle radius.
% Unless the particle diameter is given, the ratio of drum radius over
% particle radius is assumed to equal 36.

% Theodoros-Nestor Papapetrou
% Last edited: 06 Nov 2020

%% Input

% P: uint8 H x W matrix with values corresponding to colors:
% 1: red, 2: white, 0: background (black)
% P represents an image of the rotating drum with red and white particles

% d_p: the diameter of the balls in the rotating drum (in pixels),
% the same for both kinds of balls (scalar). If it is not given, a default
% value of 1/36 of the drum radius (calculated inside the function) is
% used

%% Output

% x_left: the x-coordinate of the left edge point (scalar, in pixels)

% x_right: the x-coordinate of the right edge point (scalar, in pixels)

% y_left: the y-coordinate of the left edge point (scalar, in pixels)

% y_right: the y-coordinate of the right edge point (scalar, in pixels)

% RR: the drum radius (scalar, in pixels)

%% Extract the coordinates of the free (top) and bottom surface
%[H,W] = size(masks);

[X, Ytop] = topSurface(P);
[~, Ybot] = bottomSurface(P);

%% Find drum circle
drum_circle = drumCircle(P);
x0 = drum_circle(1); % center x-coordinate
y0 = drum_circle(2); % center y-coordinate
RR = drum_circle(3); % circle radius

if nargin < 2
    ratio = 36; % drum radius over particle radius
    r = RR / ratio; % particle radius
else
    r = d_p / 2;
end

%% Find left point

found_left = 0;
middle_index = ceil(numel(X)/2);

for ii = middle_index:-1:1
    x = X(ii);
    y = Ytop(ii);
    distance = abs(norm([x - x0, y - y0]) - RR);
    if distance < r
        found_left = 1;
        break
    end
end

if ~found_left
    
    for ii = 1:middle_index
        x = X(ii);
        y = Ybot(ii);
        distance = abs(norm([x - x0, y - y0]) - RR);
        if distance < r
            break
        end
    end
    
end

x_left = x;
y_left = y;

%% Find right point

found_right = 0;
middle_index = floor(numel(X)/2);
last_index = numel(X);

for ii = middle_index:last_index
    x = X(ii);
    y = Ytop(ii);
    distance = abs(norm([x - x0, y - y0]) - RR);
    if distance < r
        found_right = 1;
        break
    end
end

if ~found_right
	
    for ii = last_index:-1:middle_index
        x = X(ii);
        y = Ybot(ii);
        distance = abs(norm([x - x0, y - y0]) - RR);
        if distance < r
            break
        end
    end
    
end

x_right = x;
y_right = y;
